@@ -125,6 +125,8 @@
 #endif
 
 #include <string.h>
+#include <math.h>
+
 #include <gobject/gvaluecollector.h>
 
 #include "clutter-animator.h"
@@ -276,6 +278,7 @@ property_iter_new (ClutterAnimator *animator,
                    PropObjectKey   *key,
                    GType            type)
 {
+  ClutterAnimatorPrivate *priv = animator->priv;
   PropertyIter *property_iter = g_slice_new (PropertyIter);
   ClutterInterval *interval = g_object_new (CLUTTER_TYPE_INTERVAL,
                                             "value-type", type,
@@ -287,8 +290,7 @@ property_iter_new (ClutterAnimator *animator,
   property_iter->interval = interval;
   property_iter->key = key;
   property_iter->alpha = clutter_alpha_new ();
-  clutter_alpha_set_timeline (property_iter->alpha,
-                              animator->priv->slave_timeline);
+  clutter_alpha_set_timeline (property_iter->alpha, priv->slave_timeline);
 
   /* as well as the alpha */
   g_object_ref_sink (property_iter->alpha);
@@ -861,6 +863,7 @@ clutter_animator_compute_value (ClutterAnimator *animator,
                                 gdouble          progress,
                                 GValue          *value)
 {
+  ClutterAnimatorPrivate *priv;
   ClutterAnimatorKey   key;
   ClutterAnimatorKey  *previous;
   ClutterAnimatorKey  *next = NULL;
@@ -870,12 +873,14 @@ clutter_animator_compute_value (ClutterAnimator *animator,
   GList               *next_l;
   gboolean             ease_in;
   ClutterInterpolation interpolation;
- 
+
   g_return_val_if_fail (CLUTTER_IS_ANIMATOR (animator), FALSE);
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (property_name, FALSE);
   g_return_val_if_fail (value, FALSE);
-  
+
+  priv = animator->priv;
+
   ease_in = clutter_animator_property_get_ease_in (animator, object,
                                                    property_name);
   interpolation = clutter_animator_property_get_interpolation (animator,
@@ -891,7 +896,7 @@ clutter_animator_compute_value (ClutterAnimator *animator,
 
   initial_l = g_list_find_custom (animator->priv->score, &key,
                                   sort_actor_prop_func);
-  if (!initial_l)
+  if (initial_l == NULL)
     return FALSE;
 
   /* first find the interval we belong in, that is the first interval
@@ -914,9 +919,7 @@ clutter_animator_compute_value (ClutterAnimator *animator,
              }
          }
        else
-         {
-           next = NULL;
-         }
+         next = NULL;
 
        if (progress < previous->progress)
          {
@@ -933,13 +936,14 @@ clutter_animator_compute_value (ClutterAnimator *animator,
             g_value_copy (&previous->value, value);
             return TRUE;
          }
+
        if (next && next->progress >= progress)
          {
             ClutterInterval *interval;
             ClutterAlpha    *alpha;
 
             gdouble sub_progress = (progress - previous->progress)
-                                    / (next->progress - previous->progress);
+                                 / (next->progress - previous->progress);
             /* this should be our interval */
             interval = g_object_new (CLUTTER_TYPE_INTERVAL,
                                      "value-type", pspec->value_type,
@@ -954,17 +958,15 @@ clutter_animator_compute_value (ClutterAnimator *animator,
                 g_value_unset (&tmp_value);
               }
             else
-              {
-                clutter_interval_set_initial_value (interval, &previous->value);
-              }
+              clutter_interval_set_initial_value (interval, &previous->value);
+
             clutter_interval_set_final_value (interval, &next->value);
 
             alpha = clutter_alpha_new ();
-            clutter_alpha_set_timeline (alpha,
-                                        animator->priv->slave_timeline);
+            clutter_alpha_set_timeline (alpha, priv->slave_timeline);
             clutter_alpha_set_mode (alpha, next->mode);
 
-            clutter_timeline_advance (animator->priv->slave_timeline,
+            clutter_timeline_advance (priv->slave_timeline,
                                       sub_progress * 10000);
             sub_progress = clutter_alpha_get_alpha (alpha);
 
@@ -1217,14 +1219,9 @@ clutter_animator_set (ClutterAnimator *animator,
           break;
         }
 
-#if GLIB_CHECK_VERSION (2, 23, 2)
       G_VALUE_COLLECT_INIT (&value, G_PARAM_SPEC_VALUE_TYPE (pspec),
                             args, 0,
                             &error);
-#else
-      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      G_VALUE_COLLECT (&value, args, 0, &error);
-#endif /* GLIB_CHECK_VERSION (2, 23, 2) */
 
       if (error)
         {
@@ -1767,7 +1764,6 @@ static void
 clutter_animator_class_init (ClutterAnimatorClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (ClutterAnimatorPrivate));
 
@@ -1784,14 +1780,13 @@ clutter_animator_class_init (ClutterAnimatorClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_uint ("duration",
-                             P_("Duration"),
-                             P_("The duration of the animation"),
-                             0, G_MAXUINT,
-                             2000,
-                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_DURATION] = pspec;
-  g_object_class_install_property (gobject_class, PROP_DURATION, pspec);
+  obj_props[PROP_DURATION] =
+    g_param_spec_uint ("duration",
+                       P_("Duration"),
+                       P_("The duration of the animation"),
+                       0, G_MAXUINT,
+                       2000,
+                       CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterAnimator:timeline:
@@ -1801,13 +1796,16 @@ clutter_animator_class_init (ClutterAnimatorClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_object ("timeline",
-                               P_("Timeline"),
-                               P_("The timeline of the animation"),
-                               CLUTTER_TYPE_TIMELINE,
-                               CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_TIMELINE] = pspec;
-  g_object_class_install_property (gobject_class, PROP_TIMELINE, pspec);
+  obj_props[PROP_TIMELINE] =
+    g_param_spec_object ("timeline",
+                         P_("Timeline"),
+                         P_("The timeline of the animation"),
+                         CLUTTER_TYPE_TIMELINE,
+                         CLUTTER_PARAM_READWRITE);
+
+  _clutter_object_class_install_properties (gobject_class,
+                                            PROP_LAST,
+                                            obj_props);
 }
 
 static void
@@ -1989,19 +1987,9 @@ clutter_animator_property_set_interpolation (ClutterAnimator     *animator,
     }
 }
 
-GType
-clutter_animator_key_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (!our_type)
-    our_type = g_boxed_type_register_static (I_("ClutterAnimatorKey"),
-                                             clutter_animator_key_copy,
-                                             clutter_animator_key_free);
-
-  return our_type;
-}
-
+G_DEFINE_BOXED_TYPE (ClutterAnimatorKey, clutter_animator_key,
+                     clutter_animator_key_copy,
+                     clutter_animator_key_free);
 
 /**
  * clutter_animator_key_get_object:
