@@ -26,7 +26,10 @@
 
 /**
  * SECTION:clutter-behaviour-path
+ * @Title: ClutterBehaviourPath
  * @short_description: A behaviour for moving actors along a #ClutterPath
+ * @Deprecated: 1.6: Use #ClutterPathConstraint and clutter_actor_animate()
+ *   with the #ClutterPathConstraint:offset property instead.
  *
  * #ClutterBehaviourPath interpolates actors along a defined path.
  *
@@ -51,8 +54,8 @@
  * ]|
  *
  * <note>If the alpha function is a periodic function, i.e. it returns to
- * 0 after reaching %CLUTTER_ALPHA_MAX_ALPHA, then the actors will walk
- * the path back to the starting #ClutterKnot.</note>
+ * 0.0 after reaching 1.0, then the actors will walk the path back to the
+ * starting #ClutterKnot.</note>
  *
  * #ClutterBehaviourPath is available since Clutter 0.2
  */
@@ -61,9 +64,9 @@
 #include "config.h"
 #endif
 
-#include "clutter-actor.h"
-#include "clutter-behaviour.h"
+#undef CLUTTER_DISABLE_DEPRECATED
 #include "clutter-behaviour-path.h"
+
 #include "clutter-bezier.h"
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
@@ -107,9 +110,13 @@ enum
 {
   PROP_0,
 
-  PROP_PATH
+  PROP_PATH,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 static void
 actor_apply_knot_foreach (ClutterBehaviour *behaviour,
                           ClutterActor     *actor,
@@ -210,22 +217,25 @@ clutter_behaviour_path_class_init (ClutterBehaviourPathClass *klass)
   gobject_class->dispose = clutter_behaviour_path_dispose;
 
   pspec = g_param_spec_object ("path",
-                               "Path",
-                               "The ClutterPath object representing the path "
-                               "to animate along",
+                               P_("Path"),
+                               P_("The ClutterPath object representing the path "
+                                  "to animate along"),
                                CLUTTER_TYPE_PATH,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_PATH] = pspec;
   g_object_class_install_property (gobject_class, PROP_PATH, pspec);
 
   /**
    * ClutterBehaviourPath::knot-reached:
    * @pathb: the object which received the signal
-   * @knot_num: the index of the #ClutterPathKnot reached
+   * @knot_num: the index of the #ClutterKnot reached
    *
    * This signal is emitted each time a node defined inside the path
    * is reached.
    *
    * Since: 0.2
+   *
+   * Deprecated: 1.6
    */
   path_signals[KNOT_REACHED] =
     g_signal_new ("knot-reached",
@@ -305,7 +315,7 @@ clutter_behaviour_path_init (ClutterBehaviourPath *self)
 
 /**
  * clutter_behaviour_path_new:
- * @alpha: a #ClutterAlpha, or %NULL
+ * @alpha: (allow-none): a #ClutterAlpha instance, or %NULL
  * @path: a #ClutterPath or %NULL for an empty path
  *
  * Creates a new path behaviour. You can use this behaviour to drive
@@ -314,9 +324,15 @@ clutter_behaviour_path_init (ClutterBehaviourPath *self)
  * This will claim the floating reference on the #ClutterPath so you
  * do not need to unref if it.
  *
- * Return value: a #ClutterBehaviour
+ * If @alpha is not %NULL, the #ClutterBehaviour will take ownership
+ * of the #ClutterAlpha instance. In the case when @alpha is %NULL,
+ * it can be set later with clutter_behaviour_set_alpha().
+ *
+ * Return value: (transfer full): a #ClutterBehaviour
  *
  * Since: 0.2
+ *
+ * Deprecated: 1.6
  */
 ClutterBehaviour *
 clutter_behaviour_path_new (ClutterAlpha *alpha,
@@ -330,15 +346,21 @@ clutter_behaviour_path_new (ClutterAlpha *alpha,
 
 /**
  * clutter_behaviour_path_new_with_description:
- * @alpha: a #ClutterAlpha
+ * @alpha: (allow-none): a #ClutterAlpha instance, or %NULL
  * @desc: a string description of the path
  *
  * Creates a new path behaviour using the path described by @desc. See
  * clutter_path_add_string() for a description of the format.
  *
- * Return value: a #ClutterBehaviour
+ * If @alpha is not %NULL, the #ClutterBehaviour will take ownership
+ * of the #ClutterAlpha instance. In the case when @alpha is %NULL,
+ * it can be set later with clutter_behaviour_set_alpha().
+ *
+ * Return value: (transfer full): a #ClutterBehaviour
  *
  * Since: 1.0
+ *
+ * Deprecated: 1.6
  */
 ClutterBehaviour *
 clutter_behaviour_path_new_with_description (ClutterAlpha *alpha,
@@ -352,7 +374,7 @@ clutter_behaviour_path_new_with_description (ClutterAlpha *alpha,
 
 /**
  * clutter_behaviour_path_new_with_knots:
- * @alpha: a #ClutterAlpha
+ * @alpha: (allow-none): a #ClutterAlpha instance, or %NULL
  * @knots: an array of #ClutterKnot<!-- -->s
  * @n_knots: number of entries in @knots
  *
@@ -363,9 +385,15 @@ clutter_behaviour_path_new_with_description (ClutterAlpha *alpha,
  * %CLUTTER_PATH_MOVE_TO and the subsequent knots are used in
  * %CLUTTER_PATH_LINE_TO<!-- -->s.
  *
- * Return value: a #ClutterBehaviour
+ * If @alpha is not %NULL, the #ClutterBehaviour will take ownership
+ * of the #ClutterAlpha instance. In the case when @alpha is %NULL,
+ * it can be set later with clutter_behaviour_set_alpha().
+ *
+ * Return value: (transfer full): a #ClutterBehaviour
  *
  * Since: 1.0
+ *
+ * Deprecated: 1.6
  */
 ClutterBehaviour *
 clutter_behaviour_path_new_with_knots (ClutterAlpha      *alpha,
@@ -399,6 +427,8 @@ clutter_behaviour_path_new_with_knots (ClutterAlpha      *alpha,
  * it.
  *
  * Since: 1.0
+ *
+ * Deprecated: 1.6
  */
 void
 clutter_behaviour_path_set_path (ClutterBehaviourPath *pathb,
@@ -418,7 +448,7 @@ clutter_behaviour_path_set_path (ClutterBehaviourPath *pathb,
 
   priv->path = path;
 
-  g_object_notify (G_OBJECT (pathb), "path");
+  _clutter_notify_by_pspec (G_OBJECT (pathb), obj_props[PROP_PATH]);
 }
 
 /**
@@ -430,6 +460,8 @@ clutter_behaviour_path_set_path (ClutterBehaviourPath *pathb,
  * Return value: (transfer none): the path
  *
  * Since: 1.0
+ *
+ * Deprecated: 1.6
  */
 ClutterPath *
 clutter_behaviour_path_get_path (ClutterBehaviourPath *pathb)
