@@ -503,16 +503,11 @@ clutter_deform_effect_class_init (ClutterDeformEffectClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
   ClutterOffscreenEffectClass *offscreen_class = CLUTTER_OFFSCREEN_EFFECT_CLASS (klass);
-  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (ClutterDeformEffectPrivate));
 
   klass->deform_vertex = clutter_deform_effect_real_deform_vertex;
 
-  gobject_class->set_property = clutter_deform_effect_set_property;
-  gobject_class->get_property = clutter_deform_effect_get_property;
-  gobject_class->finalize = clutter_deform_effect_finalize;
-
   /**
    * ClutterDeformEffect:x-tiles:
    *
@@ -521,14 +516,13 @@ clutter_deform_effect_class_init (ClutterDeformEffectClass *klass)
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_uint ("x-tiles",
-                             P_("Horiontal Tiles"),
-                             P_("The number of horizontal tiles"),
-                             1, G_MAXUINT,
-                             DEFAULT_N_TILES,
-                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_X_TILES] = pspec;
-  g_object_class_install_property (gobject_class, PROP_X_TILES, pspec);
+  obj_props[PROP_X_TILES] =
+    g_param_spec_uint ("x-tiles",
+                       P_("Horizontal Tiles"),
+                       P_("The number of horizontal tiles"),
+                       1, G_MAXUINT,
+                       DEFAULT_N_TILES,
+                       CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterDeformEffect:y-tiles:
@@ -538,14 +532,13 @@ clutter_deform_effect_class_init (ClutterDeformEffectClass *klass)
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_uint ("y-tiles",
-                             P_("Vertical Tiles"),
-                             P_("The number of vertical tiles"),
-                             1, G_MAXUINT,
-                             DEFAULT_N_TILES,
-                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_Y_TILES] = pspec;
-  g_object_class_install_property (gobject_class, PROP_Y_TILES, pspec);
+  obj_props[PROP_Y_TILES] =
+    g_param_spec_uint ("y-tiles",
+                       P_("Vertical Tiles"),
+                       P_("The number of vertical tiles"),
+                       1, G_MAXUINT,
+                       DEFAULT_N_TILES,
+                       CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterDeformEffect:back-material:
@@ -557,13 +550,19 @@ clutter_deform_effect_class_init (ClutterDeformEffectClass *klass)
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_boxed ("back-material",
-                              P_("Back Material"),
-                              P_("The material to be used when painting the back of the actor"),
-                              COGL_TYPE_HANDLE,
-                              CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_BACK_MATERIAL] = pspec;
-  g_object_class_install_property (gobject_class, PROP_BACK_MATERIAL, pspec);
+  obj_props[PROP_BACK_MATERIAL] =
+    g_param_spec_boxed ("back-material",
+                        P_("Back Material"),
+                        P_("The material to be used when painting the back of the actor"),
+                        COGL_TYPE_HANDLE,
+                        CLUTTER_PARAM_READWRITE);
+
+  gobject_class->finalize = clutter_deform_effect_finalize;
+  gobject_class->set_property = clutter_deform_effect_set_property;
+  gobject_class->get_property = clutter_deform_effect_get_property;
+  _clutter_object_class_install_properties (gobject_class,
+                                            PROP_LAST,
+                                            obj_props);
 
   meta_class->set_actor = clutter_deform_effect_set_actor;
 
