@@ -35,10 +35,13 @@
 #include "config.h"
 #endif
 
+#include "clutter-actor-private.h"
 #include "clutter-debug.h"
+#include "clutter-device-manager-private.h"
 #include "clutter-enum-types.h"
 #include "clutter-input-device.h"
 #include "clutter-private.h"
+#include "clutter-stage-private.h"
 
 enum
 {
@@ -46,9 +49,13 @@ enum
 
   PROP_ID,
   PROP_DEVICE_TYPE,
-  PROP_NAME
+  PROP_NAME,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 G_DEFINE_TYPE (ClutterInputDevice, clutter_input_device, G_TYPE_OBJECT);
 
 static void
@@ -124,12 +131,13 @@ clutter_input_device_class_init (ClutterInputDeviceClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_int ("id",
-                            "Id",
-                            "Unique identifier of the device",
+                            P_("Id"),
+                            P_("Unique identifier of the device"),
                             -1, G_MAXINT,
                             0,
                             CLUTTER_PARAM_READWRITE |
                             G_PARAM_CONSTRUCT_ONLY);
+  obj_props[PROP_ID] = pspec;
   g_object_class_install_property (gobject_class, PROP_ID, pspec);
 
   /**
@@ -140,11 +148,12 @@ clutter_input_device_class_init (ClutterInputDeviceClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_string ("name",
-                               "Name",
-                               "The name of the device",
+                               P_("Name"),
+                               P_("The name of the device"),
                                NULL,
                                CLUTTER_PARAM_READWRITE |
                                G_PARAM_CONSTRUCT_ONLY);
+  obj_props[PROP_NAME] = pspec;
   g_object_class_install_property (gobject_class, PROP_NAME, pspec);
 
   /**
@@ -155,12 +164,13 @@ clutter_input_device_class_init (ClutterInputDeviceClass *klass)
    * Since: 1.2
    */
   pspec = g_param_spec_enum ("device-type",
-                             "Device Type",
-                             "The type of the device",
+                             P_("Device Type"),
+                             P_("The type of the device"),
                              CLUTTER_TYPE_INPUT_DEVICE_TYPE,
                              CLUTTER_POINTER_DEVICE,
                              CLUTTER_PARAM_READWRITE |
                              G_PARAM_CONSTRUCT_ONLY);
+  obj_props[PROP_DEVICE_TYPE] = pspec;
   g_object_class_install_property (gobject_class, PROP_DEVICE_TYPE, pspec);
 }
 
