@@ -241,7 +241,7 @@ clutter_score_class_init (ClutterScoreClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterScoreClass, timeline_started),
 		  NULL, NULL,
-		  clutter_marshal_VOID__OBJECT,
+		  _clutter_marshal_VOID__OBJECT,
 		  G_TYPE_NONE,
 		  1, CLUTTER_TYPE_TIMELINE);
   /**
@@ -260,7 +260,7 @@ clutter_score_class_init (ClutterScoreClass *klass)
                  G_SIGNAL_RUN_LAST,
                  G_STRUCT_OFFSET (ClutterScoreClass, timeline_completed),
                  NULL, NULL,
-                 clutter_marshal_VOID__OBJECT,
+                 _clutter_marshal_VOID__OBJECT,
                  G_TYPE_NONE, 1,
                  CLUTTER_TYPE_TIMELINE);
   /**
@@ -277,7 +277,7 @@ clutter_score_class_init (ClutterScoreClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterScoreClass, completed),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   /**
    * ClutterScore::started:
@@ -293,7 +293,7 @@ clutter_score_class_init (ClutterScoreClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterScoreClass, started),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   /**
    * ClutterScore::paused:
@@ -310,7 +310,7 @@ clutter_score_class_init (ClutterScoreClass *klass)
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterScoreClass, paused),
 		  NULL, NULL,
-		  clutter_marshal_VOID__VOID,
+		  _clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
 }
 
@@ -860,7 +860,7 @@ clutter_score_clear (ClutterScore *score)
 /**
  * clutter_score_append:
  * @score: a #ClutterScore
- * @parent: a #ClutterTimeline in the score, or %NULL
+ * @parent: (allow-none): a #ClutterTimeline in the score, or %NULL
  * @timeline: a #ClutterTimeline
  *
  * Appends a timeline to another one existing in the score; the newly
@@ -1109,10 +1109,10 @@ clutter_score_get_timeline (ClutterScore *score,
  *
  * Retrieves a list of all the #ClutterTimelines managed by @score.
  *
- * Return value: (transfer container) (element-type ClutterTimeline): a #GSList
- *   containing all the timelines in the score. This function does not increase
- *   the reference count of the returned timelines. Use g_slist_free() on the
- *   returned list to deallocate its resources.
+ * Return value: (transfer container) (element-type Clutter.Timeline): a
+ *   #GSList containing all the timelines in the score. This function does
+ *   not increase the reference count of the returned timelines. Use
+ *   g_slist_free() on the returned list to deallocate its resources.
  *
  * Since: 0.6
  */
