@@ -63,11 +63,6 @@ static GList *clutter_shaders_list = NULL;
 
 #define CLUTTER_SHADER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_SHADER, ClutterShaderPrivate))
 
-typedef enum {
-  CLUTTER_VERTEX_SHADER,
-  CLUTTER_FRAGMENT_SHADER
-} ClutterShaderType;
-
 struct _ClutterShaderPrivate
 {
   guint       compiled         : 1; /* Shader is bound to the GL context */
@@ -91,9 +86,13 @@ enum
   PROP_VERTEX_SOURCE,
   PROP_FRAGMENT_SOURCE,
   PROP_COMPILED,
-  PROP_ENABLED
+  PROP_ENABLED,
+
+  PROP_LAST
 };
 
+static GParamSpec *obj_props[PROP_LAST];
+
 G_DEFINE_TYPE (ClutterShader, clutter_shader, G_TYPE_OBJECT);
 
 static void
@@ -213,10 +212,11 @@ clutter_shader_class_init (ClutterShaderClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_string ("vertex-source",
-                               "Vertex Source",
-                               "Source of vertex shader",
+                               P_("Vertex Source"),
+                               P_("Source of vertex shader"),
                                NULL,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_VERTEX_SOURCE] = pspec;
   g_object_class_install_property (object_class, PROP_VERTEX_SOURCE, pspec);
 
   /**
@@ -227,10 +227,11 @@ clutter_shader_class_init (ClutterShaderClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_string ("fragment-source",
-                               "Fragment Source",
-                               "Source of fragment shader",
+                               P_("Fragment Source"),
+                               P_("Source of fragment shader"),
                                NULL,
                                CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_FRAGMENT_SOURCE] = pspec;
   g_object_class_install_property (object_class, PROP_FRAGMENT_SOURCE, pspec);
 
   /**
@@ -242,10 +243,11 @@ clutter_shader_class_init (ClutterShaderClass *klass)
    * Since: 0.8
    */
   pspec = g_param_spec_boolean ("compiled",
-                                "Compiled",
-                                "Whether the shader is compiled and linked",
+                                P_("Compiled"),
+                                P_("Whether the shader is compiled and linked"),
                                 FALSE,
                                 CLUTTER_PARAM_READABLE);
+  obj_props[PROP_COMPILED] = pspec;
   g_object_class_install_property (object_class, PROP_COMPILED, pspec);
 
   /**
@@ -256,10 +258,11 @@ clutter_shader_class_init (ClutterShaderClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_boolean ("enabled",
-                                "Enabled",
-                                "Whether the shader is enabled",
+                                P_("Enabled"),
+                                P_("Whether the shader is enabled"),
                                 FALSE,
                                 CLUTTER_PARAM_READWRITE);
+  obj_props[PROP_ENABLED] = pspec;
   g_object_class_install_property (object_class, PROP_ENABLED, pspec);
 }
 
@@ -330,7 +333,7 @@ clutter_shader_set_source (ClutterShader     *shader,
 
       priv->fragment_source = g_strndup (data, length);
       priv->fragment_is_glsl = is_glsl;
-      g_object_notify (G_OBJECT (shader), "fragment-source");
+      _clutter_notify_by_pspec (G_OBJECT (shader), obj_props[PROP_FRAGMENT_SOURCE]);
       break;
 
     case CLUTTER_VERTEX_SHADER:
@@ -338,7 +341,7 @@ clutter_shader_set_source (ClutterShader     *shader,
 
       priv->vertex_source = g_strndup (data, length);
       priv->vertex_is_glsl = is_glsl;
-      g_object_notify (G_OBJECT (shader), "vertex-source");
+      _clutter_notify_by_pspec (G_OBJECT (shader), obj_props[PROP_VERTEX_SOURCE]);
       break;
     }
 
@@ -565,7 +568,7 @@ clutter_shader_compile (ClutterShader  *shader,
     }
 
   priv->compiled = bind_glsl_shader (shader, error);
-  g_object_notify (G_OBJECT (shader), "compiled");
+  _clutter_notify_by_pspec (G_OBJECT (shader), obj_props[PROP_COMPILED]);
 
   return priv->compiled;
 }
@@ -606,7 +609,7 @@ clutter_shader_release (ClutterShader *shader)
   priv->program = COGL_INVALID_HANDLE;
   priv->compiled = FALSE;
 
-  g_object_notify (G_OBJECT (shader), "compiled");
+  _clutter_notify_by_pspec (G_OBJECT (shader), obj_props[PROP_COMPILED]);
 }
 
 /**
@@ -674,7 +677,7 @@ clutter_shader_set_is_enabled (ClutterShader *shader,
       else
         cogl_program_use (COGL_INVALID_HANDLE);
 
-      g_object_notify (G_OBJECT (shader), "enabled");
+      _clutter_notify_by_pspec (G_OBJECT (shader), obj_props[PROP_ENABLED]);
     }
 }
 
@@ -736,33 +739,38 @@ clutter_shader_set_uniform (ClutterShader *shader,
       const GLfloat *floats;
 
       floats = clutter_value_get_shader_float (value, &size);
-      cogl_program_uniform_float (location, size, 1, floats);
+      cogl_program_set_uniform_float (priv->program,
+                                      location, size, 1, floats);
     }
   else if (CLUTTER_VALUE_HOLDS_SHADER_INT (value))
     {
       const int *ints;
 
       ints = clutter_value_get_shader_int (value, &size);
-      cogl_program_uniform_int (location, size, 1, ints);
+      cogl_program_set_uniform_int (priv->program,
+                                    location, size, 1, ints);
     }
   else if (CLUTTER_VALUE_HOLDS_SHADER_MATRIX (value))
     {
       const GLfloat *matrix;
 
       matrix = clutter_value_get_shader_matrix (value, &size);
-      cogl_program_uniform_matrix (location, size, 1, FALSE, matrix);
+      cogl_program_set_uniform_matrix (priv->program,
+                                       location, size, 1, FALSE, matrix);
     }
   else if (G_VALUE_HOLDS_FLOAT (value))
     {
       GLfloat float_val = g_value_get_float (value);
 
-      cogl_program_uniform_float (location, 1, 1, &float_val);
+      cogl_program_set_uniform_float (priv->program,
+                                      location, 1, 1, &float_val);
     }
   else if (G_VALUE_HOLDS_INT (value))
     {
       int int_val = g_value_get_int (value);
 
-      cogl_program_uniform_int (location, 1, 1, &int_val);
+      cogl_program_set_uniform_int (priv->program,
+                                    location, 1, 1, &int_val);
     }
   else
     g_assert_not_reached ();
@@ -828,7 +836,9 @@ clutter_shader_get_vertex_source (ClutterShader *shader)
  *
  * Retrieves the underlying #CoglHandle for the shader program.
  *
- * Return value: A #CoglHandle for the shader program, or %NULL
+ * Return value: (transfer none): A #CoglHandle for the shader program,
+ *   or %NULL. The handle is owned by the #ClutterShader and it should
+ *   not be unreferenced
  *
  * Since: 1.0
  */
@@ -846,7 +856,9 @@ clutter_shader_get_cogl_program (ClutterShader *shader)
  *
  * Retrieves the underlying #CoglHandle for the fragment shader.
  *
- * Return value: A #CoglHandle for the fragment shader, or %NULL
+ * Return value: (transfer none): A #CoglHandle for the fragment
+ *   shader, or %NULL. The handle is owned by the #ClutterShader
+ *   and it should not be unreferenced
  *
  * Since: 1.0
  */
@@ -864,7 +876,9 @@ clutter_shader_get_cogl_fragment_shader (ClutterShader *shader)
  *
  * Retrieves the underlying #CoglHandle for the vertex shader.
  *
- * Return value: A #CoglHandle for the vertex shader, or %NULL
+ * Return value: (transfer none): A #CoglHandle for the vertex
+ *   shader, or %NULL. The handle is owned by the #ClutterShader
+ *   and it should not be unreferenced
  *
  * Since: 1.0
  */
