@@ -55,22 +55,24 @@
 
 #include "clutter-stage.h"
 
-#include "clutter-backend.h"
-#include "clutter-main.h"
+#include "clutter-actor-private.h"
+#include "clutter-backend-private.h"
 #include "clutter-color.h"
-#include "clutter-util.h"
+#include "clutter-container.h"
+#include "clutter-debug.h"
+#include "clutter-device-manager-private.h"
+#include "clutter-enum-types.h"
+#include "clutter-id-pool.h"
+#include "clutter-main.h"
 #include "clutter-marshal.h"
 #include "clutter-master-clock.h"
-#include "clutter-enum-types.h"
+#include "clutter-paint-volume-private.h"
 #include "clutter-private.h"
-#include "clutter-debug.h"
-#include "clutter-stage-manager.h"
-#include "clutter-stage-window.h"
-#include "clutter-version.h" 	/* For flavour */
-#include "clutter-id-pool.h"
-#include "clutter-container.h"
 #include "clutter-profile.h"
-#include "clutter-input-device.h"
+#include "clutter-stage-manager-private.h"
+#include "clutter-stage-private.h"
+#include "clutter-util.h"
+#include "clutter-version.h" 	/* For flavour */
 
 #include "cogl/cogl.h"
 
@@ -98,6 +100,13 @@ typedef enum { /*< prefix=CLUTTER_STAGE >*/
 
 #define STAGE_NO_CLEAR_ON_PAINT(s)      ((((ClutterStage *) (s))->priv->stage_hints & CLUTTER_STAGE_NO_CLEAR_ON_PAINT) != 0)
 
+struct _ClutterStageQueueRedrawEntry
+{
+  ClutterActor *actor;
+  gboolean has_clip;
+  ClutterPaintVolume clip;
+};
+
 struct _ClutterStagePrivate
 {
   /* the stage implementation */
@@ -118,8 +127,15 @@ struct _ClutterStagePrivate
 
   gint                picks_per_frame;
 
+  GArray             *paint_volume_stack;
+
+  const ClutterGeometry *current_paint_clip;
+
+  GList              *pending_queue_redraws;
+
   ClutterPickMode     pick_buffer_mode;
 
+  guint relayout_pending       : 1;
   guint redraw_pending         : 1;
   guint is_fullscreen          : 1;
   guint is_cursor_visible      : 1;
@@ -166,6 +182,11 @@ static guint stage_signals[LAST_SIGNAL] = { 0, };
 
 static const ClutterColor default_stage_color = { 255, 255, 255, 255 };
 
+static void _clutter_stage_maybe_finish_queue_redraws (ClutterStage *stage);
+
+static void
+_clutter_stage_maybe_finish_queue_redraws (ClutterStage *stage);
+
 static void
 clutter_stage_get_preferred_width (ClutterActor *self,
                                    gfloat        for_height,
@@ -207,12 +228,35 @@ clutter_stage_get_preferred_height (ClutterActor *self,
   if (natural_height_p)
     *natural_height_p = geom.height;
 }
+
+static inline void
+queue_full_redraw (ClutterStage *stage)
+{
+  ClutterStageWindow *stage_window;
+
+  if (CLUTTER_ACTOR_IN_DESTRUCTION (stage))
+    return;
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
+
+  /* Just calling clutter_actor_queue_redraw will typically only
+   * redraw the bounding box of the children parented on the stage but
+   * in this case we really need to ensure that the full stage is
+   * redrawn so we add a NULL redraw clip to the stage window. */
+  stage_window = _clutter_stage_get_window (stage);
+  if (stage_window == NULL)
+    return;
+
+  _clutter_stage_window_add_redraw_clip (stage_window, NULL);
+}
+
 static void
 clutter_stage_allocate (ClutterActor           *self,
                         const ClutterActorBox  *box,
                         ClutterAllocationFlags  flags)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+  ClutterGeometry prev_geom;
   ClutterGeometry geom = { 0, };
   gboolean origin_changed;
   gint width, height;
@@ -222,6 +266,8 @@ clutter_stage_allocate (ClutterActor           *self,
   if (priv->impl == NULL)
     return;
 
+  clutter_actor_get_allocation_geometry (self, &prev_geom);
+
   width = clutter_actor_box_get_width (box);
   height = clutter_actor_box_get_height (box);
   _clutter_stage_window_get_geometry (priv->impl, &geom);
@@ -306,13 +352,34 @@ clutter_stage_allocate (ClutterActor           *self,
   _cogl_onscreen_clutter_backend_set_size (geom.width, geom.height);
 
   clutter_actor_get_allocation_geometry (self, &geom);
-  _clutter_stage_set_viewport (CLUTTER_STAGE (self),
-                               0, 0, geom.width, geom.height);
+  if (geom.width != prev_geom.width || geom.height != prev_geom.height)
+    {
+      _clutter_stage_set_viewport (CLUTTER_STAGE (self),
+                                   0, 0, geom.width, geom.height);
 
-  /* Note: we don't assume that set_viewport will queue a full redraw
-   * since it may bail-out early if something preemptively set the
-   * viewport before the stage was really allocated its new size. */
-  clutter_actor_queue_redraw (self);
+      /* Note: we don't assume that set_viewport will queue a full redraw
+       * since it may bail-out early if something preemptively set the
+       * viewport before the stage was really allocated its new size. */
+      queue_full_redraw (CLUTTER_STAGE (self));
+    }
+}
+
+/* This provides a common point of entry for painting the scenegraph
+ * for picking or painting...
+ *
+ * XXX: Instead of having a toplevel 2D clip region, it might be
+ * better to have a clip volume within the view frustum. This could
+ * allow us to avoid projecting actors into window coordinates to
+ * be able to cull them.
+ */
+void
+_clutter_stage_do_paint (ClutterStage *stage, const ClutterGeometry *clip)
+{
+  ClutterStagePrivate *priv = stage->priv;
+  priv->current_paint_clip = clip;
+  _clutter_stage_paint_volume_stack_free_all (stage);
+  clutter_actor_paint (CLUTTER_ACTOR (stage));
+  priv->current_paint_clip = NULL;
 }
 
 static void
@@ -388,6 +455,15 @@ clutter_stage_pick (ClutterActor       *self,
                              NULL);
 }
 
+static gboolean
+clutter_stage_get_paint_volume (ClutterActor *self,
+                                ClutterPaintVolume *volume)
+{
+  /* Returning False effectively means Clutter has to assume it covers
+   * everything... */
+  return FALSE;
+}
+
 static void
 clutter_stage_realize (ClutterActor *self)
 {
@@ -409,12 +485,6 @@ clutter_stage_realize (ClutterActor *self)
    */
   if (is_realized)
     {
-     //MAEMO5 PR-1.3 FIX.//
-     CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
-     clutter_stage_ensure_current (CLUTTER_STAGE (self));
-     return;
-     //MAEMO5 PR-1.3 FIX.//
-
       ClutterBackend *backend = clutter_get_default_backend ();
       GError *error = NULL;
 
@@ -673,7 +743,54 @@ _clutter_stage_needs_update (ClutterStage *stage)
 
   priv = stage->priv;
 
-  return priv->redraw_pending;
+  return priv->relayout_pending || priv->redraw_pending;
+}
+
+void
+_clutter_stage_maybe_relayout (ClutterActor *actor)
+{
+  ClutterStage *stage = CLUTTER_STAGE (actor);
+  ClutterStagePrivate *priv = stage->priv;
+  gfloat natural_width, natural_height;
+  ClutterActorBox box = { 0, };
+  CLUTTER_STATIC_TIMER (relayout_timer,
+                        "Mainloop", /* no parent */
+                        "Layouting",
+                        "The time spent reallocating the stage",
+                        0 /* no application private data */);
+
+  if (!priv->relayout_pending)
+    return;
+  priv->relayout_pending = FALSE;
+
+  /* avoid reentrancy */
+  if (!CLUTTER_ACTOR_IN_RELAYOUT (stage))
+    {
+      CLUTTER_TIMER_START (_clutter_uprof_context, relayout_timer);
+      CLUTTER_NOTE (ACTOR, "Recomputing layout");
+
+      CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_IN_RELAYOUT);
+
+      natural_width = natural_height = 0;
+      clutter_actor_get_preferred_size (CLUTTER_ACTOR (stage),
+                                        NULL, NULL,
+                                        &natural_width, &natural_height);
+
+      box.x1 = 0;
+      box.y1 = 0;
+      box.x2 = natural_width;
+      box.y2 = natural_height;
+
+      CLUTTER_NOTE (ACTOR, "Allocating (0, 0 - %d, %d) for the stage",
+                    (int) natural_width,
+                    (int) natural_height);
+
+      clutter_actor_allocate (CLUTTER_ACTOR (stage),
+                              &box, CLUTTER_ALLOCATION_NONE);
+
+      CLUTTER_UNSET_PRIVATE_FLAGS (stage, CLUTTER_IN_RELAYOUT);
+      CLUTTER_TIMER_STOP (_clutter_uprof_context, relayout_timer);
+    }
 }
 
 /**
@@ -693,16 +810,16 @@ _clutter_stage_do_update (ClutterStage *stage)
 
   priv = stage->priv;
 
+  /* NB: We need to ensure we have an up to date layout *before* we
+   * check or clear the pending redraws flag since a relayout may
+   * queue a redraw.
+   */
+  _clutter_stage_maybe_relayout (CLUTTER_ACTOR (stage));
+
   if (!priv->redraw_pending)
     return FALSE;
 
-  /* clutter_do_redraw() will also call maybe_relayout(), but since a relayout
-   * can queue a redraw, we want to do the relayout before we clear the
-   * update_idle to avoid painting the stage twice. Calling maybe_relayout()
-   * twice in a row is cheap because of caching of requested and allocated
-   * size.
-   */
-  _clutter_stage_maybe_relayout (CLUTTER_ACTOR (stage));
+  _clutter_stage_maybe_finish_queue_redraws (stage);
 
   CLUTTER_NOTE (PAINT, "redrawing via idle for stage[%p]", stage);
   _clutter_do_redraw (stage);
@@ -721,6 +838,20 @@ _clutter_stage_do_update (ClutterStage *stage)
   return TRUE;
 }
 
+static void
+clutter_stage_real_queue_relayout (ClutterActor *self)
+{
+  ClutterStage *stage = CLUTTER_STAGE (self);
+  ClutterStagePrivate *priv = stage->priv;
+  ClutterActorClass *parent_class;
+
+  priv->relayout_pending = TRUE;
+
+  /* chain up */
+  parent_class = CLUTTER_ACTOR_CLASS (clutter_stage_parent_class);
+  parent_class->queue_relayout (self);
+}
+
 static void
 clutter_stage_real_queue_redraw (ClutterActor *actor,
                                  ClutterActor *leaf)
@@ -729,79 +860,66 @@ clutter_stage_real_queue_redraw (ClutterActor *actor,
   ClutterStagePrivate *priv = stage->priv;
   ClutterStageWindow *stage_window;
   ClutterGeometry stage_clip;
-  const ClutterActorBox *clip;
-  ClutterActorBox bounds;
-  ClutterVertex v[4];
-  int i;
+  const ClutterPaintVolume *redraw_clip;
+  ClutterPaintVolume projected_clip;
+  CoglMatrix modelview;
+  ClutterActorBox bounding_box;
 
-  CLUTTER_NOTE (PAINT, "Redraw request number %lu",
-                CLUTTER_CONTEXT ()->redraw_count + 1);
-
-  if (!priv->redraw_pending)
-    {
-      ClutterMasterClock *master_clock;
-
-      priv->redraw_pending = TRUE;
-
-      master_clock = _clutter_master_clock_get_default ();
-      _clutter_master_clock_start_running (master_clock);
-    }
-  else
-    CLUTTER_CONTEXT ()->redraw_count += 1;
-
-  /* We have an optimization in _clutter_do_pick to detect when the
-   * scene is static so we can cache a full, un-clipped pick buffer to
-   * avoid continuous pick renders.
-   *
-   * Currently the assumption is that actors queue a redraw when some
-   * state changes that affects painting *or* picking so we can use
-   * this point to invalidate any currently cached pick buffer.
-   */
-  _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
+  if (CLUTTER_ACTOR_IN_DESTRUCTION (actor))
+    return;
 
   /* If the backend can't do anything with redraw clips (e.g. it already knows
    * it needs to redraw everything anyway) then don't spend time transforming
-   * any clip regions into stage coordinates... */
+   * any clip volume into stage coordinates... */
   stage_window = _clutter_stage_get_window (stage);
-  if (_clutter_stage_window_ignoring_redraw_clips (stage_window))
+  if (stage_window == NULL)
     return;
 
-  /* Convert the clip rectangle (which is in leaf actor coordinates) into stage
-   * coordinates and then into an axis aligned stage coordinates bounding
-   * box...
-   */
-
-  clip = _clutter_actor_get_queue_redraw_clip (leaf);
-  if (!clip)
+  if (_clutter_stage_window_ignoring_redraw_clips (stage_window))
     {
       _clutter_stage_window_add_redraw_clip (stage_window, NULL);
       return;
     }
 
-  _clutter_actor_transform_and_project_box (leaf, clip, v);
+  /* Convert the clip volume (which is in leaf actor coordinates) into stage
+   * coordinates and then into an axis aligned stage coordinates bounding
+   * box...
+   */
 
-  bounds.x1 = v[0].x; bounds.y1 = v[0].y;
-  bounds.x2 = v[0].x; bounds.y2 = v[0].y;
-
-  for (i = 0; i < 4; i++)
+  if (!_clutter_actor_get_queue_redraw_clip (leaf))
     {
-      if (v[i].x < bounds.x1)
-        bounds.x1 = v[i].x;
-      else if (v[i].x > bounds.x2)
-        bounds.x2 = v[i].x;
-
-      if (v[i].y < bounds.y1)
-        bounds.y1 = v[i].y;
-      else if (v[i].y > bounds.y2)
-        bounds.y2 = v[i].y;
+      _clutter_stage_window_add_redraw_clip (stage_window, NULL);
+      return;
     }
 
+  redraw_clip = _clutter_actor_get_queue_redraw_clip (leaf);
+
+  _clutter_paint_volume_copy_static (redraw_clip, &projected_clip);
+
+  /* NB: _clutter_actor_apply_modelview_transform_recursive will never
+   * include the transformation between stage coordinates and OpenGL
+   * window coordinates, we have to explicitly use the
+   * stage->apply_transform to get that... */
+  cogl_matrix_init_identity (&modelview);
+  _clutter_actor_apply_modelview_transform (CLUTTER_ACTOR (stage), &modelview);
+  _clutter_actor_apply_modelview_transform_recursive (leaf, NULL, &modelview);
+
+  _clutter_paint_volume_project (&projected_clip,
+                                 &modelview,
+                                 &priv->projection,
+                                 priv->viewport);
+
+  _clutter_paint_volume_get_bounding_box (&projected_clip, &bounding_box);
+  clutter_paint_volume_free (&projected_clip);
+
+  clutter_actor_box_clamp_to_pixel (&bounding_box);
+
   /* when converting to integer coordinates make sure we round the edges of the
    * clip rectangle outwards... */
-  stage_clip.x = bounds.x1;
-  stage_clip.y = bounds.y1;
-  stage_clip.width = ceilf (bounds.x2) - stage_clip.x;
-  stage_clip.height = ceilf (bounds.y2) - stage_clip.y;
+  stage_clip.x = bounding_box.x1;
+  stage_clip.y = bounding_box.y1;
+  stage_clip.width = bounding_box.x2 - stage_clip.x;
+  stage_clip.height = bounding_box.y2 - stage_clip.y;
 
   _clutter_stage_window_add_redraw_clip (stage_window, &stage_clip);
 }
@@ -1083,6 +1201,8 @@ clutter_stage_finalize (GObject *object)
 
   g_free (stage->priv->title);
 
+  g_array_free (priv->paint_volume_stack, TRUE);
+
   G_OBJECT_CLASS (clutter_stage_parent_class)->finalize (object);
 }
 
@@ -1103,10 +1223,12 @@ clutter_stage_class_init (ClutterStageClass *klass)
   actor_class->get_preferred_height = clutter_stage_get_preferred_height;
   actor_class->paint = clutter_stage_paint;
   actor_class->pick = clutter_stage_pick;
+  actor_class->get_paint_volume = clutter_stage_get_paint_volume;
   actor_class->realize = clutter_stage_realize;
   actor_class->unrealize = clutter_stage_unrealize;
   actor_class->show = clutter_stage_show;
   actor_class->hide = clutter_stage_hide;
+  actor_class->queue_relayout = clutter_stage_real_queue_relayout;
   actor_class->queue_redraw = clutter_stage_real_queue_redraw;
   actor_class->apply_transform = clutter_stage_real_apply_transform;
 
@@ -1466,6 +1588,8 @@ clutter_stage_init (ClutterStage *self)
   priv->fog.z_near = 1.0;
   priv->fog.z_far  = 2.0;
 
+  priv->relayout_pending = TRUE;
+
   clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
   clutter_stage_set_title (self, g_get_prgname ());
   clutter_stage_set_key_focus (self, NULL);
@@ -1480,6 +1604,9 @@ clutter_stage_init (ClutterStage *self)
 
   _clutter_stage_set_pick_buffer_valid (self, FALSE, CLUTTER_PICK_ALL);
   _clutter_stage_reset_picks_per_frame_counter (self);
+
+  priv->paint_volume_stack =
+    g_array_new (FALSE, FALSE, sizeof (ClutterPaintVolume));
 }
 
 /**
@@ -1711,7 +1838,7 @@ _clutter_stage_set_viewport (ClutterStage *stage,
 
   priv->dirty_viewport = TRUE;
 
-  clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
+  queue_full_redraw (stage);
 }
 
 /* This simply provides a simple mechanism for us to ensure that
@@ -2443,17 +2570,9 @@ clutter_perspective_free (gpointer data)
     g_slice_free (ClutterPerspective, data);
 }
 
-GType
-clutter_perspective_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (!our_type)
-    our_type = g_boxed_type_register_static (I_("ClutterPerspective"),
-                                             clutter_perspective_copy,
-                                             clutter_perspective_free);
-  return our_type;
-}
+G_DEFINE_BOXED_TYPE (ClutterPerspective, clutter_perspective,
+                     clutter_perspective_copy,
+                     clutter_perspective_free);
 
 static gpointer
 clutter_fog_copy (gpointer data)
@@ -2471,18 +2590,7 @@ clutter_fog_free (gpointer data)
     g_slice_free (ClutterFog, data);
 }
 
-GType
-clutter_fog_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (G_UNLIKELY (our_type == 0))
-    our_type = g_boxed_type_register_static (I_("ClutterFog"),
-                                             clutter_fog_copy,
-                                             clutter_fog_free);
-
-  return our_type;
-}
+G_DEFINE_BOXED_TYPE (ClutterFog, clutter_fog, clutter_fog_copy, clutter_fog_free);
 
 /**
  * clutter_stage_new:
@@ -2491,7 +2599,7 @@ clutter_fog_get_type (void)
  * top-level actor which can be used as another container. It works
  * exactly like the default stage, but while clutter_stage_get_default()
  * will always return the same instance, you will have to keep a pointer
- * to any #ClutterStage returned by clutter_stage_create().
+ * to any #ClutterStage returned by clutter_stage_new().
  *
  * The ability to support multiple stages depends on the current
  * backend. Use clutter_feature_available() and
@@ -2609,6 +2717,7 @@ clutter_stage_ensure_redraw (ClutterStage *stage)
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
   priv = stage->priv;
+  priv->relayout_pending = TRUE;
   priv->redraw_pending = TRUE;
 
   master_clock = _clutter_master_clock_get_default ();
@@ -3003,3 +3112,192 @@ _clutter_stage_get_picks_per_frame_counter (ClutterStage *stage)
 
   return stage->priv->picks_per_frame;
 }
+
+ClutterPaintVolume *
+_clutter_stage_paint_volume_stack_allocate (ClutterStage *stage)
+{
+  GArray *paint_volume_stack = stage->priv->paint_volume_stack;
+
+  g_array_set_size (paint_volume_stack,
+                    paint_volume_stack->len+1);
+
+  return &g_array_index (paint_volume_stack,
+                         ClutterPaintVolume,
+                         paint_volume_stack->len - 1);
+}
+
+void
+_clutter_stage_paint_volume_stack_free_all (ClutterStage *stage)
+{
+  GArray *paint_volume_stack = stage->priv->paint_volume_stack;
+  int i;
+
+  for (i = 0; i < paint_volume_stack->len; i++)
+    {
+      ClutterPaintVolume *pv =
+        &g_array_index (paint_volume_stack, ClutterPaintVolume, i);
+      clutter_paint_volume_free (pv);
+    }
+
+  g_array_set_size (paint_volume_stack, 0);
+}
+
+/* The is an out-of-band paramater available while painting that
+ * can be used to cull actors. */
+const ClutterGeometry *
+_clutter_stage_get_clip (ClutterStage *stage)
+{
+  return stage->priv->current_paint_clip;
+}
+
+/* When an actor queues a redraw we add it to a list on the stage that
+ * gets processed once all updates to the stage have been finished.
+ *
+ * This deferred approach to processing queue_redraw requests means
+ * that we can avoid redundant transformations of clip volumes if
+ * something later triggers a full stage redraw anyway. It also means
+ * we can be more sure that all the referenced actors will have valid
+ * allocations improving the chance that we can determine the actors
+ * paint volume so we can clip the redraw request even if the user
+ * didn't explicitly do so.
+ */
+ClutterStageQueueRedrawEntry *
+_clutter_stage_queue_actor_redraw (ClutterStage *stage,
+                                   ClutterStageQueueRedrawEntry *entry,
+                                   ClutterActor *actor,
+                                   ClutterPaintVolume *clip)
+{
+  ClutterStagePrivate *priv = stage->priv;
+
+  CLUTTER_NOTE (PAINT, "Redraw request number %lu",
+                CLUTTER_CONTEXT ()->redraw_count + 1);
+
+  if (!priv->redraw_pending)
+    {
+      ClutterMasterClock *master_clock;
+
+      priv->redraw_pending = TRUE;
+
+      master_clock = _clutter_master_clock_get_default ();
+      _clutter_master_clock_start_running (master_clock);
+    }
+  else
+    CLUTTER_CONTEXT ()->redraw_count += 1;
+
+  /* We have an optimization in _clutter_do_pick to detect when the
+   * scene is static so we can cache a full, un-clipped pick buffer to
+   * avoid continuous pick renders.
+   *
+   * Currently the assumption is that actors queue a redraw when some
+   * state changes that affects painting *or* picking so we can use
+   * this point to invalidate any currently cached pick buffer.
+   */
+  _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
+
+  if (entry)
+    {
+      /* Ignore all requests to queue a redraw for an actor if a full
+       * (non-clipped) redraw of the actor has already been queued. */
+      if (!entry->has_clip)
+        return entry;
+
+      /* If queuing a clipped redraw and a clipped redraw has
+       * previously been queued for this actor then combine the latest
+       * clip together with the existing clip */
+      if (clip)
+        clutter_paint_volume_union (&entry->clip, clip);
+      else
+        {
+          clutter_paint_volume_free (&entry->clip);
+          entry->has_clip = FALSE;
+        }
+      return entry;
+    }
+  else
+    {
+      entry = g_slice_new (ClutterStageQueueRedrawEntry);
+      entry->actor = g_object_ref (actor);
+
+      if (clip)
+        {
+          entry->has_clip = TRUE;
+          _clutter_paint_volume_init_static (actor, &entry->clip);
+          _clutter_paint_volume_set_from_volume (&entry->clip, clip);
+        }
+      else
+        entry->has_clip = FALSE;
+
+      stage->priv->pending_queue_redraws =
+        g_list_prepend (stage->priv->pending_queue_redraws, entry);
+
+      return entry;
+    }
+}
+
+static void
+free_queue_redraw_entry (ClutterStageQueueRedrawEntry *entry)
+{
+  if (entry->actor)
+    g_object_unref (entry->actor);
+  if (entry->has_clip)
+    clutter_paint_volume_free (&entry->clip);
+  g_slice_free (ClutterStageQueueRedrawEntry, entry);
+}
+
+void
+_clutter_stage_queue_redraw_entry_invalidate (ClutterStageQueueRedrawEntry *entry)
+{
+  if (entry == NULL)
+    return;
+
+  if (entry->actor != NULL)
+    {
+      g_object_unref (entry->actor);
+      entry->actor = NULL;
+    }
+
+  if (entry->has_clip)
+    {
+      clutter_paint_volume_free (&entry->clip);
+      entry->has_clip = FALSE;
+    }
+}
+
+static void
+_clutter_stage_maybe_finish_queue_redraws (ClutterStage *stage)
+{
+  /* Note: we have to repeat until the pending_queue_redraws list is
+   * empty because actors are allowed to queue redraws in response to
+   * the queue-redraw signal. For example Clone actors or
+   * texture_new_from_actor actors will have to queue a redraw if
+   * their source queues a redraw.
+   */
+  while (stage->priv->pending_queue_redraws)
+    {
+      GList *l;
+      /* XXX: we need to allow stage->priv->pending_queue_redraws to
+       * be updated while we process the current entries in the list
+       * so we steal the list pointer and then reset it to an empty
+       * list before processing... */
+      GList *stolen_list = stage->priv->pending_queue_redraws;
+      stage->priv->pending_queue_redraws = NULL;
+
+      for (l = stolen_list; l; l = l->next)
+        {
+          ClutterStageQueueRedrawEntry *entry = l->data;
+          ClutterPaintVolume *clip;
+
+          /* NB: Entries may be invalidated if the actor gets destroyed */
+          if (G_LIKELY (entry->actor != NULL))
+	    {
+	      clip = entry->has_clip ? &entry->clip : NULL;
+
+	      _clutter_actor_finish_queue_redraw (entry->actor, clip);
+	    }
+
+          free_queue_redraw_entry (entry);
+        }
+      g_list_free (stolen_list);
+    }
+}
+
