@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * Authors:
  *   Robert Bragg <robert@linux.intel.com>
@@ -103,7 +103,6 @@ static CoglBlendStringColorSourceInfo tex_combine_texture_n_color_source = {
 /* NB: These must be sorted so any name that's a subset of another
  * comes later than the longer name. */
 static CoglBlendStringFunctionInfo tex_combine_functions[] = {
-  DEFINE_FUNCTION (AUTO_COMPOSITE, 14, 0),
   DEFINE_FUNCTION (REPLACE, 7, 1),
   DEFINE_FUNCTION (MODULATE, 8, 2),
   DEFINE_FUNCTION (ADD_SIGNED, 10, 2),
@@ -115,7 +114,6 @@ static CoglBlendStringFunctionInfo tex_combine_functions[] = {
 };
 
 static CoglBlendStringFunctionInfo blend_functions[] = {
-  DEFINE_FUNCTION (AUTO_COMPOSITE, 14, 0),
   DEFINE_FUNCTION (ADD, 3, 2)
 };
 
@@ -172,15 +170,6 @@ validate_tex_combine_statements (CoglBlendStringStatement *statements,
 
   for (i = 0; i < n_statements; i++)
     {
-#ifdef HAVE_COGL_GLES2
-      if (statements[i].function->type != COGL_BLEND_STRING_FUNCTION_MODULATE)
-        {
-          error_string = "Using anything but MODULATE() for texture combining"
-                         " under GLES 2 is currently unsupported";
-          detail = COGL_BLEND_STRING_ERROR_GPU_UNSUPPORTED_ERROR;
-          goto error;
-        }
-#endif
       for (j = 0; j < statements[i].function->argc; j++)
         {
           CoglBlendStringArgument *arg = &statements[i].args[j];
@@ -196,16 +185,6 @@ validate_tex_combine_statements (CoglBlendStringStatement *statements,
                              "not texture combining";
               goto error;
             }
-#ifdef HAVE_COGL_GLES2
-          if (arg->source.info->type == COGL_BLEND_STRING_COLOR_SOURCE_CONSTANT)
-            {
-              error_string = "Using a constant for texture combining isn't "
-                             "currently supported with GLES 2 "
-                             "(TODO: glTexEnvf)";
-              detail = COGL_BLEND_STRING_ERROR_GPU_UNSUPPORTED_ERROR;
-              goto error;
-            }
-#endif
         }
     }
 
