@@ -123,21 +123,19 @@ convert_bind_target_to_gl_target (CoglBufferBindTarget target)
         return GL_PIXEL_UNPACK_BUFFER;
       case COGL_BUFFER_BIND_TARGET_VERTEX_ARRAY:
         return GL_ARRAY_BUFFER;
-      case COGL_BUFFER_BIND_TARGET_VERTEX_INDICES_ARRAY:
+      case COGL_BUFFER_BIND_TARGET_INDEX_ARRAY:
         return GL_ELEMENT_ARRAY_BUFFER;
       default:
         g_return_val_if_reached (COGL_BUFFER_BIND_TARGET_PIXEL_UNPACK);
     }
 }
 
-static guint8 *
+static void *
 bo_map (CoglBuffer       *buffer,
         CoglBufferAccess  access,
         CoglBufferMapHint hints)
 {
 #ifndef COGL_HAS_GLES
-
-  CoglPixelArray *pixel_array = COGL_PIXEL_ARRAY (buffer);
   guint8 *data;
   CoglBufferBindTarget target;
   GLenum gl_target;
@@ -152,21 +150,20 @@ bo_map (CoglBuffer       *buffer,
   /* create an empty store if we don't have one yet. creating the store
    * lazily allows the user of the CoglBuffer to set a hint before the
    * store is created. */
-  if (!COGL_PIXEL_ARRAY_FLAG_IS_SET (pixel_array, STORE_CREATED) ||
-      (hints & COGL_BUFFER_MAP_HINT_DISCARD))
+  if (!buffer->store_created || (hints & COGL_BUFFER_MAP_HINT_DISCARD))
     {
       GE( glBufferData (gl_target,
                         buffer->size,
                         NULL,
                         _cogl_buffer_hints_to_gl_enum (buffer->usage_hint,
                                                        buffer->update_hint)) );
-      COGL_PIXEL_ARRAY_SET_FLAG (pixel_array, STORE_CREATED);
+      buffer->store_created = TRUE;
     }
 
   GE_RET( data, glMapBuffer (gl_target,
                              _cogl_buffer_access_to_gl_enum (access)) );
   if (data)
-    COGL_BUFFER_SET_FLAG (buffer, MAPPED);
+    buffer->flags |= COGL_BUFFER_FLAG_MAPPED;
 
   _cogl_buffer_unbind (buffer);
 
@@ -188,7 +185,7 @@ bo_unmap (CoglBuffer *buffer)
   _cogl_buffer_bind (buffer, buffer->last_target);
 
   GE( glUnmapBuffer (convert_bind_target_to_gl_target (buffer->last_target)) );
-  COGL_BUFFER_CLEAR_FLAG (buffer, MAPPED);
+  buffer->flags &= ~COGL_BUFFER_FLAG_MAPPED;
 
   _cogl_buffer_unbind (buffer);
 #else
@@ -199,10 +196,9 @@ bo_unmap (CoglBuffer *buffer)
 static gboolean
 bo_set_data (CoglBuffer   *buffer,
              unsigned int  offset,
-             const guint8 *data,
+             const void   *data,
              unsigned int  size)
 {
-  CoglPixelArray *pixel_array = COGL_PIXEL_ARRAY (buffer);
   CoglBufferBindTarget target;
   GLenum gl_target;
 
@@ -216,14 +212,14 @@ bo_set_data (CoglBuffer   *buffer,
   /* create an empty store if we don't have one yet. creating the store
    * lazily allows the user of the CoglBuffer to set a hint before the
    * store is created. */
-  if (!COGL_PIXEL_ARRAY_FLAG_IS_SET (pixel_array, STORE_CREATED))
+  if (!buffer->store_created)
     {
       GE( glBufferData (gl_target,
                         buffer->size,
                         NULL,
                         _cogl_buffer_hints_to_gl_enum (buffer->usage_hint,
                                                        buffer->update_hint)) );
-      COGL_PIXEL_ARRAY_SET_FLAG (pixel_array, STORE_CREATED);
+      buffer->store_created = TRUE;
     }
 
   GE( glBufferSubData (gl_target, offset, size, data) );
@@ -237,25 +233,25 @@ bo_set_data (CoglBuffer   *buffer,
  * Fallback path, buffer->data points to a malloc'ed buffer.
  */
 
-static guint8 *
+static void *
 malloc_map (CoglBuffer       *buffer,
             CoglBufferAccess  access,
             CoglBufferMapHint hints)
 {
-  COGL_BUFFER_SET_FLAG (buffer, MAPPED);
+  buffer->flags |= COGL_BUFFER_FLAG_MAPPED;
   return buffer->data;
 }
 
 static void
 malloc_unmap (CoglBuffer *buffer)
 {
-  COGL_BUFFER_CLEAR_FLAG (buffer, MAPPED);
+  buffer->flags &= ~COGL_BUFFER_FLAG_MAPPED;
 }
 
 static gboolean
 malloc_set_data (CoglBuffer   *buffer,
                  unsigned int  offset,
-                 const guint8 *data,
+                 const void   *data,
                  unsigned int  size)
 {
   memcpy (buffer->data + offset, data, size);
@@ -272,12 +268,14 @@ _cogl_buffer_initialize (CoglBuffer           *buffer,
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  buffer->flags       = COGL_BUFFER_FLAG_NONE;
-  buffer->size        = size;
-  buffer->last_target = default_target;
-  buffer->usage_hint  = usage_hint;
-  buffer->update_hint = update_hint;
-  buffer->data        = NULL;
+  buffer->flags         = COGL_BUFFER_FLAG_NONE;
+  buffer->store_created = FALSE;
+  buffer->size          = size;
+  buffer->last_target   = default_target;
+  buffer->usage_hint    = usage_hint;
+  buffer->update_hint   = update_hint;
+  buffer->data          = NULL;
+  buffer->immutable_ref = 0;
 
   if (use_malloc)
     {
@@ -294,14 +292,22 @@ _cogl_buffer_initialize (CoglBuffer           *buffer,
       buffer->vtable.set_data = bo_set_data;
 
       GE( glGenBuffers (1, &buffer->gl_handle) );
-      COGL_BUFFER_SET_FLAG (buffer, BUFFER_OBJECT);
+      buffer->flags |= COGL_BUFFER_FLAG_BUFFER_OBJECT;
     }
 }
 
 void
 _cogl_buffer_fini (CoglBuffer *buffer)
 {
-  g_return_if_fail (!COGL_BUFFER_FLAG_IS_SET (buffer, MAPPED));
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  g_return_if_fail (!(buffer->flags & COGL_BUFFER_FLAG_MAPPED));
+  g_return_if_fail (buffer->immutable_ref == 0);
+
+  if (buffer->flags & COGL_BUFFER_FLAG_BUFFER_OBJECT)
+    GE( glDeleteBuffers (1, &buffer->gl_handle) );
+  else
+    g_free (buffer->data);
 }
 
 /* OpenGL ES 1.1 and 2 have a GL_OES_mapbuffer extension that is able to map
@@ -354,28 +360,31 @@ _cogl_buffer_hints_to_gl_enum (CoglBufferUsageHint  usage_hint,
 }
 #endif
 
-void
+void *
 _cogl_buffer_bind (CoglBuffer *buffer, CoglBufferBindTarget target)
 {
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  _COGL_GET_CONTEXT (ctx, NULL);
 
-  g_return_if_fail (buffer != NULL);
+  g_return_val_if_fail (buffer != NULL, NULL);
 
   /* Don't allow binding the buffer to multiple targets at the same time */
-  g_return_if_fail (ctx->current_buffer[buffer->last_target] != buffer);
+  g_return_val_if_fail (ctx->current_buffer[buffer->last_target] != buffer,
+                        NULL);
 
   /* Don't allow nesting binds to the same target */
-  g_return_if_fail (ctx->current_buffer[target] == NULL);
+  g_return_val_if_fail (ctx->current_buffer[target] == NULL, NULL);
 
   buffer->last_target = target;
+  ctx->current_buffer[target] = buffer;
 
-  if (COGL_BUFFER_FLAG_IS_SET (buffer, BUFFER_OBJECT))
+  if (buffer->flags & COGL_BUFFER_FLAG_BUFFER_OBJECT)
     {
       GLenum gl_target = convert_bind_target_to_gl_target (buffer->last_target);
       GE( glBindBuffer (gl_target, buffer->gl_handle) );
+      return NULL;
     }
-
-  ctx->current_buffer[target] = buffer;
+  else
+    return buffer->data;
 }
 
 void
@@ -388,7 +397,7 @@ _cogl_buffer_unbind (CoglBuffer *buffer)
   /* the unbind should pair up with a previous bind */
   g_return_if_fail (ctx->current_buffer[buffer->last_target] == buffer);
 
-  if (COGL_BUFFER_FLAG_IS_SET (buffer, BUFFER_OBJECT))
+  if (buffer->flags & COGL_BUFFER_FLAG_BUFFER_OBJECT)
     {
       GLenum gl_target = convert_bind_target_to_gl_target (buffer->last_target);
       GE( glBindBuffer (gl_target, 0) );
@@ -428,15 +437,29 @@ cogl_buffer_get_update_hint (CoglBuffer *buffer)
   return buffer->update_hint;
 }
 
-guint8 *
+static void
+warn_about_midscene_changes (void)
+{
+  static gboolean seen = FALSE;
+  if (!seen)
+    {
+      g_warning ("Mid-scene modification of buffers has "
+                 "undefined results\n");
+      seen = TRUE;
+    }
+}
+
+void *
 cogl_buffer_map (CoglBuffer        *buffer,
                  CoglBufferAccess   access,
                  CoglBufferMapHint  hints)
 {
-  if (!cogl_is_buffer (buffer))
-    return NULL;
+  g_return_val_if_fail (cogl_is_buffer (buffer), NULL);
 
-  if (COGL_BUFFER_FLAG_IS_SET (buffer, MAPPED))
+  if (G_UNLIKELY (buffer->immutable_ref))
+    warn_about_midscene_changes ();
+
+  if (buffer->flags & COGL_BUFFER_FLAG_MAPPED)
     return buffer->data;
 
   buffer->data = buffer->vtable.map (buffer, access, hints);
@@ -449,7 +472,7 @@ cogl_buffer_unmap (CoglBuffer *buffer)
   if (!cogl_is_buffer (buffer))
     return;
 
-  if (!COGL_BUFFER_FLAG_IS_SET (buffer, MAPPED))
+  if (!(buffer->flags & COGL_BUFFER_FLAG_MAPPED))
     return;
 
   buffer->vtable.unmap (buffer);
@@ -458,14 +481,33 @@ cogl_buffer_unmap (CoglBuffer *buffer)
 gboolean
 cogl_buffer_set_data (CoglBuffer   *buffer,
                       gsize         offset,
-                      const guint8 *data,
+                      const void   *data,
                       gsize         size)
 {
-  if (!cogl_is_buffer (buffer))
-    return FALSE;
+  g_return_val_if_fail (cogl_is_buffer (buffer), FALSE);
+  g_return_val_if_fail ((offset + size) <= buffer->size, FALSE);
 
-  if (G_UNLIKELY((offset + size) > buffer->size))
-    return FALSE;
+  if (G_UNLIKELY (buffer->immutable_ref))
+    warn_about_midscene_changes ();
 
   return buffer->vtable.set_data (buffer, offset, data, size);
 }
+
+CoglBuffer *
+_cogl_buffer_immutable_ref (CoglBuffer *buffer)
+{
+  g_return_val_if_fail (cogl_is_buffer (buffer), NULL);
+
+  buffer->immutable_ref++;
+  return buffer;
+}
+
+void
+_cogl_buffer_immutable_unref (CoglBuffer *buffer)
+{
+  g_return_if_fail (cogl_is_buffer (buffer));
+  g_return_if_fail (buffer->immutable_ref > 0);
+
+  buffer->immutable_ref--;
+}
+
