@@ -25,12 +25,20 @@
 #define __COGL_CONTEXT_H
 
 #include "cogl-internal.h"
-#include "cogl-context-driver.h"
+
+#if HAVE_COGL_GL
+#include "cogl-context-driver-gl.h"
+#endif
+
+#if HAVE_COGL_GLES || HAVE_COGL_GLES2
+#include "cogl-context-driver-gles.h"
+#endif
+
 #include "cogl-context-winsys.h"
 #include "cogl-primitives.h"
 #include "cogl-clip-stack.h"
 #include "cogl-matrix-stack.h"
-#include "cogl-material-private.h"
+#include "cogl-pipeline-private.h"
 #include "cogl-buffer-private.h"
 #include "cogl-bitmask.h"
 #include "cogl-atlas.h"
@@ -47,9 +55,8 @@ typedef struct
   /* Features cache */
   CoglFeatureFlags        feature_flags;
   CoglFeatureFlagsPrivate feature_flags_private;
-  gboolean                features_cached;
 
-  CoglHandle        default_material;
+  CoglHandle        default_pipeline;
   CoglHandle        default_layer_0;
   CoglHandle        default_layer_n;
   CoglHandle        dummy_layer_dependant;
@@ -72,13 +79,15 @@ typedef struct
   GArray           *texture_units;
   int               active_texture_unit;
 
-  CoglMaterialFogState legacy_fog_state;
+  CoglPipelineFogState legacy_fog_state;
 
-  /* Materials */
-  CoglMaterial     *simple_material; /* used for set_source_color */
-  CoglMaterial     *texture_material; /* used for set_source_texture */
-  CoglMaterial     *source_material;
-  GString          *arbfp_source_buffer;
+  /* Pipelines */
+  CoglPipeline     *opaque_color_pipeline; /* used for set_source_color */
+  CoglPipeline     *blended_color_pipeline; /* used for set_source_color */
+  CoglPipeline     *texture_pipeline; /* used for set_source_texture */
+  GString          *fragment_header_buffer;
+  GString          *fragment_source_buffer;
+  GList            *source_stack;
 
   int               legacy_state_set;
 
@@ -92,15 +101,18 @@ typedef struct
    * can batch things together. */
   GArray           *journal;
   GArray           *logged_vertices;
+  GArray           *journal_flush_attributes_array;
+
   GArray           *polygon_vertices;
 
   /* Some simple caching, to minimize state changes... */
-  CoglMaterial     *current_material;
-  unsigned long     current_material_changes_since_flush;
-  gboolean          current_material_skip_gl_color;
+  CoglPipeline     *current_pipeline;
+  unsigned long     current_pipeline_changes_since_flush;
+  gboolean          current_pipeline_skip_gl_color;
+  unsigned long     current_pipeline_age;
 
-  GArray           *material0_nodes;
-  GArray           *material1_nodes;
+  GArray           *pipeline0_nodes;
+  GArray           *pipeline1_nodes;
 
   /* Bitmask of texture coordinates arrays that are enabled */
   CoglBitmask       texcoord_arrays_enabled;
@@ -131,18 +143,22 @@ typedef struct
   gboolean          dirty_gl_viewport;
 
   /* Primitives */
-  CoglHandle        current_path;
-  CoglMaterial     *stencil_material;
+  CoglPath         *current_path;
+  CoglPipeline     *stencil_pipeline;
 
   /* Pre-generated VBOs containing indices to generate GL_TRIANGLES
      out of a vertex array of quads */
-  CoglHandle        quad_indices_byte;
-  unsigned int      quad_indices_short_len;
-  CoglHandle        quad_indices_short;
+  CoglHandle        quad_buffer_indices_byte;
+  unsigned int      quad_buffer_indices_len;
+  CoglHandle        quad_buffer_indices;
+
+  CoglIndices      *rectangle_byte_indices;
+  CoglIndices      *rectangle_short_indices;
+  int               rectangle_short_indices_len;
 
   gboolean          in_begin_gl_block;
 
-  CoglMaterial     *texture_download_material;
+  CoglPipeline     *texture_download_pipeline;
 
   CoglAtlas        *atlas;
 
@@ -161,7 +177,7 @@ typedef struct
   /* Fragment processing programs */
   CoglHandle              current_program;
 
-  CoglMaterialProgramType current_use_program_type;
+  CoglPipelineProgramType current_use_program_type;
   GLuint                  current_gl_program;
 
   /* List of types that will be considered a subclass of CoglTexture in
@@ -172,6 +188,28 @@ typedef struct
      cogl_is_buffer */
   GSList           *buffer_types;
 
+  /* Clipping */
+  /* TRUE if we have a valid clipping stack flushed. In that case
+     current_clip_stack will describe what the current state is. If
+     this is FALSE then the current clip stack is completely unknown
+     so it will need to be reflushed. In that case current_clip_stack
+     doesn't need to be a valid pointer. We can't just use NULL in
+     current_clip_stack to mark a dirty state because NULL is a valid
+     stack (meaning no clipping) */
+  gboolean          current_clip_stack_valid;
+  /* The clip state that was flushed. This isn't intended to be used
+     as a stack to push and pop new entries. Instead the current stack
+     that the user wants is part of the framebuffer state. This is
+     just used to record the flush state so we can avoid flushing the
+     same state multiple times. When the clip state is flushed this
+     will hold a reference */
+  CoglClipStack    *current_clip_stack;
+  /* Whether the stencil buffer was used as part of the current clip
+     state. If TRUE then any further use of the stencil buffer (such
+     as for drawing paths) would need to be merged with the existing
+     stencil buffer */
+  gboolean          current_clip_stack_uses_stencil;
+
   CoglContextDriver drv;
   CoglContextWinsys winsys;
 } CoglContext;
