@@ -139,12 +139,14 @@ _cogl_is_framebuffer (void *object)
 static void
 _cogl_framebuffer_init (CoglFramebuffer *framebuffer,
                         CoglFramebufferType type,
+                        CoglPixelFormat format,
                         int width,
                         int height)
 {
   framebuffer->type             = type;
   framebuffer->width            = width;
   framebuffer->height           = height;
+  framebuffer->format           = format;
   framebuffer->viewport_x       = 0;
   framebuffer->viewport_y       = 0;
   framebuffer->viewport_width   = width;
@@ -324,22 +326,31 @@ _cogl_framebuffer_init_bits (CoglFramebuffer *framebuffer)
   framebuffer->dirty_bitmasks = FALSE;
 }
 
+typedef struct
+{
+  CoglHandle texture;
+  unsigned int level;
+  unsigned int level_width;
+  unsigned int level_height;
+} CoglFramebufferTryFBOData;
+
 static gboolean
-try_creating_fbo (TryFBOFlags flags,
-                  CoglHandle texture,
-		  GSList **renderbuffers_return,
-		  GLuint *fbo_handle_return)
+try_creating_fbo (CoglOffscreen *offscreen,
+                  TryFBOFlags flags,
+                  CoglFramebufferTryFBOData *data)
 {
   GLuint gl_depth_stencil_handle;
   GLuint gl_depth_handle;
   GLuint gl_stencil_handle;
   GLuint tex_gl_handle;
   GLenum tex_gl_target;
+  GLuint fbo_gl_handle;
   GLenum status;
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  if (!cogl_texture_get_gl_texture (texture, &tex_gl_handle, &tex_gl_target))
+  if (!cogl_texture_get_gl_texture (data->texture,
+                                    &tex_gl_handle, &tex_gl_target))
     return FALSE;
 
   if (tex_gl_target != GL_TEXTURE_2D
@@ -357,11 +368,12 @@ try_creating_fbo (TryFBOFlags flags,
   ctx->dirty_bound_framebuffer = 1;
 
   /* Generate framebuffer */
-  glGenFramebuffers (1, fbo_handle_return);
-  GE (glBindFramebuffer (GL_FRAMEBUFFER, *fbo_handle_return));
+  glGenFramebuffers (1, &fbo_gl_handle);
+  GE (glBindFramebuffer (GL_FRAMEBUFFER, fbo_gl_handle));
+  offscreen->fbo_handle = fbo_gl_handle;
 
   GE (glFramebufferTexture2D (GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-			      tex_gl_target, tex_gl_handle, 0));
+                              tex_gl_target, tex_gl_handle, data->level));
 
   if (flags & _TRY_DEPTH_STENCIL)
     {
@@ -369,8 +381,8 @@ try_creating_fbo (TryFBOFlags flags,
       GE (glGenRenderbuffers (1, &gl_depth_stencil_handle));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, gl_depth_stencil_handle));
       GE (glRenderbufferStorage (GL_RENDERBUFFER, GL_DEPTH_STENCIL,
-                                 cogl_texture_get_width (texture),
-                                 cogl_texture_get_height (texture)));
+                                 data->level_width,
+                                 data->level_height));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, 0));
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_STENCIL_ATTACHMENT,
@@ -378,8 +390,8 @@ try_creating_fbo (TryFBOFlags flags,
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_DEPTH_ATTACHMENT,
                                      GL_RENDERBUFFER, gl_depth_stencil_handle));
-      *renderbuffers_return =
-        g_slist_prepend (*renderbuffers_return,
+      offscreen->renderbuffers =
+        g_slist_prepend (offscreen->renderbuffers,
                          GUINT_TO_POINTER (gl_depth_stencil_handle));
     }
 
@@ -390,14 +402,14 @@ try_creating_fbo (TryFBOFlags flags,
       /* For now we just ask for GL_DEPTH_COMPONENT16 since this is all that's
        * available under GLES */
       GE (glRenderbufferStorage (GL_RENDERBUFFER, GL_DEPTH_COMPONENT16,
-                                 cogl_texture_get_width (texture),
-                                 cogl_texture_get_height (texture)));
+                                 data->level_width,
+                                 data->level_height));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, 0));
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_DEPTH_ATTACHMENT,
                                      GL_RENDERBUFFER, gl_depth_handle));
-      *renderbuffers_return =
-        g_slist_prepend (*renderbuffers_return,
+      offscreen->renderbuffers =
+        g_slist_prepend (offscreen->renderbuffers,
                          GUINT_TO_POINTER (gl_depth_handle));
     }
 
@@ -406,14 +418,14 @@ try_creating_fbo (TryFBOFlags flags,
       GE (glGenRenderbuffers (1, &gl_stencil_handle));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, gl_stencil_handle));
       GE (glRenderbufferStorage (GL_RENDERBUFFER, GL_STENCIL_INDEX8,
-                                 cogl_texture_get_width (texture),
-                                 cogl_texture_get_height (texture)));
+                                 data->level_width,
+                                 data->level_height));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, 0));
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_STENCIL_ATTACHMENT,
                                      GL_RENDERBUFFER, gl_stencil_handle));
-      *renderbuffers_return =
-        g_slist_prepend (*renderbuffers_return,
+      offscreen->renderbuffers =
+        g_slist_prepend (offscreen->renderbuffers,
                          GUINT_TO_POINTER (gl_stencil_handle));
     }
 
@@ -424,16 +436,16 @@ try_creating_fbo (TryFBOFlags flags,
     {
       GSList *l;
 
-      GE (glDeleteFramebuffers (1, fbo_handle_return));
+      GE (glDeleteFramebuffers (1, &fbo_gl_handle));
 
-      for (l = *renderbuffers_return; l; l = l->next)
+      for (l = offscreen->renderbuffers; l; l = l->next)
         {
           GLuint renderbuffer = GPOINTER_TO_UINT (l->data);
           GE (glDeleteRenderbuffers (1, &renderbuffer));
         }
 
-      g_slist_free (*renderbuffers_return);
-      *renderbuffers_return = NULL;
+      g_slist_free (offscreen->renderbuffers);
+      offscreen->renderbuffers = NULL;
 
       return FALSE;
     }
@@ -441,44 +453,17 @@ try_creating_fbo (TryFBOFlags flags,
   return TRUE;
 }
 
-static gboolean
-try_creating_fbo_alternatives (CoglHandle texhandle,
-			       GSList **renderbuffers_return,
-			       GLuint *fbo_handle_return)
-{
-  static TryFBOFlags  flags;
-  static gboolean     have_working_flags = FALSE;
-
-  if ((have_working_flags &&
-       try_creating_fbo (flags, texhandle,
-                         renderbuffers_return, fbo_handle_return)) ||
-#ifdef HAVE_COGL_GL
-      try_creating_fbo (flags = _TRY_DEPTH_STENCIL, texhandle,
-                        renderbuffers_return, fbo_handle_return) ||
-#endif
-      try_creating_fbo (flags = _TRY_DEPTH | _TRY_STENCIL, texhandle,
-                        renderbuffers_return, fbo_handle_return) ||
-      try_creating_fbo (flags = _TRY_STENCIL, texhandle,
-                        renderbuffers_return, fbo_handle_return) ||
-      try_creating_fbo (flags = _TRY_DEPTH, texhandle,
-                        renderbuffers_return, fbo_handle_return) ||
-      try_creating_fbo (flags = 0, texhandle,
-                        renderbuffers_return, fbo_handle_return))
-    {
-      /* Record that the last set of flags succeeded so that we can
-         try that set first next time */
-      have_working_flags = TRUE;
-
-      return TRUE;
-    }
-
-  return TRUE;
-}
-
 CoglHandle
-cogl_offscreen_new_to_texture (CoglHandle texhandle)
+_cogl_offscreen_new_to_texture_full (CoglHandle texhandle,
+                                     CoglOffscreenFlags create_flags,
+                                     unsigned int level)
 {
   CoglOffscreen      *offscreen;
+  static TryFBOFlags  flags;
+  static gboolean     have_working_flags = FALSE;
+  unsigned int        i;
+  CoglFramebufferTryFBOData data;
+  gboolean            fbo_created;
 
   _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
 
@@ -493,6 +478,27 @@ cogl_offscreen_new_to_texture (CoglHandle texhandle)
   if (cogl_texture_is_sliced (texhandle))
     return COGL_INVALID_HANDLE;
 
+  data.texture = texhandle;
+  data.level = level;
+
+  /* Calculate the size of the texture at this mipmap level to ensure
+     that it's a valid level */
+  data.level_width = cogl_texture_get_width (texhandle);
+  data.level_height = cogl_texture_get_height (texhandle);
+
+  for (i = 0; i < level; i++)
+    {
+      /* If neither dimension can be further divided then the level is
+         invalid */
+      if (data.level_width == 1 && data.level_height == 1)
+        return COGL_INVALID_HANDLE;
+
+      if (data.level_width > 1)
+        data.level_width >>= 1;
+      if (data.level_height > 1)
+        data.level_height >>= 1;
+    }
+
   /* XXX: The framebuffer_object spec isn't clear in defining whether attaching
    * a texture as a renderbuffer with mipmap filtering enabled while the
    * mipmaps have not been uploaded should result in an incomplete framebuffer
@@ -501,21 +507,44 @@ cogl_offscreen_new_to_texture (CoglHandle texhandle)
    * To avoid an error with drivers that do consider this a problem we
    * explicitly set non mipmapped filters here. These will later be reset when
    * the texture is actually used for rendering according to the filters set on
-   * the corresponding CoglMaterial.
+   * the corresponding CoglPipeline.
    */
   _cogl_texture_set_filters (texhandle, GL_NEAREST, GL_NEAREST);
 
   offscreen = g_new0 (CoglOffscreen, 1);
+  offscreen->texture = cogl_handle_ref (texhandle);
 
-  if (try_creating_fbo_alternatives (texhandle, &offscreen->renderbuffers,
-                                     &offscreen->fbo_handle))
+  if ((create_flags & COGL_OFFSCREEN_DISABLE_DEPTH_AND_STENCIL))
+    fbo_created = try_creating_fbo (offscreen, 0, &data);
+  else
     {
-      offscreen->texture = cogl_handle_ref (texhandle);
+      if ((have_working_flags &&
+           try_creating_fbo (offscreen, flags, &data)) ||
+#ifdef HAVE_COGL_GL
+          try_creating_fbo (offscreen, flags = _TRY_DEPTH_STENCIL, &data) ||
+#endif
+          try_creating_fbo (offscreen, flags = _TRY_DEPTH | _TRY_STENCIL,
+                            &data) ||
+          try_creating_fbo (offscreen, flags = _TRY_STENCIL, &data) ||
+          try_creating_fbo (offscreen, flags = _TRY_DEPTH, &data) ||
+          try_creating_fbo (offscreen, flags = 0, &data))
+        {
+          /* Record that the last set of flags succeeded so that we can
+             try that set first next time */
+          have_working_flags = TRUE;
+          fbo_created = TRUE;
+        }
+      else
+        fbo_created = FALSE;
+    }
 
+  if (fbo_created)
+    {
       _cogl_framebuffer_init (COGL_FRAMEBUFFER (offscreen),
                               COGL_FRAMEBUFFER_TYPE_OFFSCREEN,
-                              cogl_texture_get_width (texhandle),
-                              cogl_texture_get_height (texhandle));
+                              cogl_texture_get_format (texhandle),
+                              data.level_width,
+                              data.level_height);
 
       return _cogl_offscreen_object_new (offscreen);
     }
@@ -528,13 +557,22 @@ cogl_offscreen_new_to_texture (CoglHandle texhandle)
     }
 }
 
+CoglHandle
+cogl_offscreen_new_to_texture (CoglHandle texhandle)
+{
+  return _cogl_offscreen_new_to_texture_full (texhandle, 0, 0);
+}
+
 static void
-free_buffers (CoglOffscreen *offscreen)
+_cogl_offscreen_free (CoglOffscreen *offscreen)
 {
   GSList *l;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  /* Chain up to parent */
+  _cogl_framebuffer_free (COGL_FRAMEBUFFER (offscreen));
+
   for (l = offscreen->renderbuffers; l; l = l->next)
     {
       GLuint renderbuffer = GPOINTER_TO_UINT (l->data);
@@ -546,85 +584,10 @@ free_buffers (CoglOffscreen *offscreen)
 
   if (offscreen->texture != COGL_INVALID_HANDLE)
     cogl_handle_unref (offscreen->texture);
-}
-
-static void
-_cogl_offscreen_free (CoglOffscreen *offscreen)
-{
-  /* Chain up to parent */
-  _cogl_framebuffer_free (COGL_FRAMEBUFFER (offscreen));
-
-  free_buffers (offscreen);
 
   g_free (offscreen);
 }
 
-gboolean
-cogl_offscreen_set_to_texture (CoglHandle handle,
-                               CoglHandle texhandle)
-{
-  CoglOffscreen *offscreen;
-  CoglFramebuffer *framebuffer;
-  int width, height;
-  GLuint fbo_handle;
-  GSList *renderbuffers = NULL;
-
-  _COGL_GET_CONTEXT (ctx, FALSE);
-  
-  if (!cogl_is_offscreen(handle))
-    return FALSE;
-
-  if (!cogl_is_texture (texhandle))
-    return FALSE;
-
-  if (cogl_texture_is_sliced (texhandle))
-    return FALSE;
-
-  _cogl_texture_set_filters (texhandle, GL_NEAREST, GL_NEAREST);
-    
-  offscreen = _cogl_offscreen_pointer_from_handle(handle);
-  framebuffer = COGL_FRAMEBUFFER (offscreen);
-
-  width = cogl_texture_get_width (texhandle);
-  height = cogl_texture_get_height (texhandle);
-
-  if (width == framebuffer->width && height == framebuffer->height)
-    {
-      GLuint tex_gl_handle;
-      GLenum tex_gl_target;
-      
-      if (!cogl_texture_get_gl_texture (texhandle, &tex_gl_handle, &tex_gl_target))
-	return FALSE;
-
-      _cogl_journal_flush ();
-      ctx->dirty_bound_framebuffer = 1;
-
-      GE (glBindFramebuffer (GL_FRAMEBUFFER, offscreen->fbo_handle));
-      GE (glFramebufferTexture2D (GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-				  tex_gl_target, tex_gl_handle, 0));
-            
-      return TRUE;
-    }
-  else if (try_creating_fbo_alternatives (texhandle, &renderbuffers, &fbo_handle))
-    {
-    
-      free_buffers (offscreen);
-      
-      offscreen->texture = cogl_handle_ref (texhandle);
-      framebuffer->width = width;
-      framebuffer->height = height;
-
-      offscreen->fbo_handle = fbo_handle;
-      offscreen->renderbuffers = renderbuffers;
-
-      framebuffer->dirty_bitmasks = TRUE;
-
-      return TRUE;
-    }
-
-  return FALSE;
-}
-
 CoglHandle
 _cogl_onscreen_new (void)
 {
@@ -634,9 +597,21 @@ _cogl_onscreen_new (void)
    * implement CoglOnscreen framebuffers, since we can't, e.g. keep track of
    * the window size. */
 
+  /* FIXME: We are assuming onscreen buffers will always be
+     premultiplied so we'll set the premult flag on the bitmap
+     format. This will usually be correct because the result of the
+     default blending operations for Cogl ends up with premultiplied
+     data in the framebuffer. However it is possible for the
+     framebuffer to be in whatever format depending on what
+     CoglPipeline is used to render to it. Eventually we may want to
+     add a way for an application to inform Cogl that the framebuffer
+     is not premultiplied in case it is being used for some special
+     purpose. */
+
   onscreen = g_new0 (CoglOnscreen, 1);
   _cogl_framebuffer_init (COGL_FRAMEBUFFER (onscreen),
                           COGL_FRAMEBUFFER_TYPE_ONSCREEN,
+                          COGL_PIXEL_FORMAT_RGBA_8888_PRE,
                           0xdeadbeef, /* width */
                           0xdeadbeef); /* height */
 
@@ -728,7 +703,7 @@ _cogl_set_framebuffer_real (CoglFramebuffer *framebuffer)
    * we flush */
   _cogl_matrix_stack_dirty (framebuffer->modelview_stack);
   _cogl_matrix_stack_dirty (framebuffer->projection_stack);
-  _cogl_clip_state_dirty (&framebuffer->clip_state);
+  _cogl_clip_stack_dirty ();
 }
 
 void
@@ -875,7 +850,8 @@ _cogl_framebuffer_flush_state (CoglFramebuffer *framebuffer,
   /* XXX: Flushing clip state may trash the modelview and projection
    * matrices so we must do it before flushing the matrices...
    */
-  _cogl_clip_state_flush (&framebuffer->clip_state);
+  if (!(flags & COGL_FRAMEBUFFER_FLUSH_SKIP_CLIP_STATE))
+    _cogl_clip_state_flush (&framebuffer->clip_state);
 
   if (!(flags & COGL_FRAMEBUFFER_FLUSH_SKIP_MODELVIEW))
     _cogl_matrix_stack_flush_to_gl (framebuffer->modelview_stack,
