@@ -32,7 +32,7 @@
 
 #include "cogl-fixed.h"
 #include "cogl-internal.h"
-#include "cogl-material.h"
+#include "cogl-pipeline.h"
 #include "cogl-offscreen.h"
 #include "cogl-shader.h"
 #include "cogl-texture.h"
