@@ -36,8 +36,8 @@
 #include "cogl-internal.h"
 #include "cogl-util.h"
 #include "cogl-context.h"
-#include "cogl-material-private.h"
-#include "cogl-material-opengl-private.h"
+#include "cogl-pipeline-private.h"
+#include "cogl-pipeline-opengl-private.h"
 #include "cogl-winsys.h"
 #include "cogl-framebuffer-private.h"
 #include "cogl-matrix-private.h"
@@ -148,20 +148,15 @@ cogl_check_extension (const char *name, const char *ext)
   return _cogl_check_extension (name, ext);
 }
 
+/* This version of cogl_clear can be used internally as an alternative
+   to avoid flushing the journal or the framebuffer state. This is
+   needed when doing operations that may be called whiling flushing
+   the journal */
 void
-cogl_clear (const CoglColor *color, unsigned long buffers)
+_cogl_clear (const CoglColor *color, unsigned long buffers)
 {
   GLbitfield gl_buffers = 0;
 
-  COGL_NOTE (DRAW, "Clear begin");
-
-  _cogl_journal_flush ();
-
-  /* NB: _cogl_framebuffer_flush_state may disrupt various state (such
-   * as the material state) when flushing the clip stack, so should
-   * always be done first when preparing to draw. */
-  _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
-
   if (buffers & COGL_BUFFER_BIT_COLOR)
     {
       GE( glClearColor (cogl_color_get_red_float (color),
@@ -201,6 +196,21 @@ cogl_clear (const CoglColor *color, unsigned long buffers)
       _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
       ctxt->journal_rectangles_color = 1;
     }
+}
+
+void
+cogl_clear (const CoglColor *color, unsigned long buffers)
+{
+  COGL_NOTE (DRAW, "Clear begin");
+
+  _cogl_journal_flush ();
+
+  /* NB: _cogl_framebuffer_flush_state may disrupt various state (such
+   * as the pipeline state) when flushing the clip stack, so should
+   * always be done first when preparing to draw. */
+  _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
+
+  _cogl_clear (color, buffers);;
 
   COGL_NOTE (DRAW, "Clear end");
 }
@@ -369,15 +379,24 @@ _cogl_flush_face_winding (void)
 void
 cogl_set_source_color (const CoglColor *color)
 {
-  CoglColor premultiplied;
+  CoglPipeline *pipeline;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  premultiplied = *color;
-  cogl_color_premultiply (&premultiplied);
-  cogl_material_set_color (ctx->simple_material, &premultiplied);
+  if (cogl_color_get_alpha_byte (color) == 0xff)
+    {
+      cogl_pipeline_set_color (ctx->opaque_color_pipeline, color);
+      pipeline = ctx->opaque_color_pipeline;
+    }
+  else
+    {
+      CoglColor premultiplied = *color;
+      cogl_color_premultiply (&premultiplied);
+      cogl_pipeline_set_color (ctx->blended_color_pipeline, &premultiplied);
+      pipeline = ctx->blended_color_pipeline;
+    }
 
-  cogl_set_source (ctx->simple_material);
+  cogl_set_source (pipeline);
 }
 
 void
@@ -408,97 +427,11 @@ cogl_viewport (unsigned int width,
   cogl_set_viewport (0, 0, width, height);
 }
 
-void
-_cogl_setup_viewport (unsigned int width,
-                      unsigned int height,
-                      float fovy,
-                      float aspect,
-                      float z_near,
-                      float z_far)
-{
-  float z_camera;
-  CoglMatrix projection_matrix;
-  CoglMatrixStack *modelview_stack;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  cogl_set_viewport (0, 0, width, height);
-
-  /* For Ortho projection.
-   * _cogl_matrix_stack_ortho (projection_stack, 0, width, 0,  height, -1, 1);
-   */
-
-  cogl_perspective (fovy, aspect, z_near, z_far);
-
-  /*
-   * In theory, we can compute the camera distance from screen as:
-   *
-   *   0.5 * tan (FOV)
-   *
-   * However, it's better to compute the z_camera from our projection
-   * matrix so that we get a 1:1 mapping at the screen distance. Consider
-   * the upper-left corner of the screen. It has object coordinates
-   * (0,0,0), so by the transform below, ends up with eye coordinate
-   *
-   *   x_eye = x_object / width - 0.5 = - 0.5
-   *   y_eye = (height - y_object) / width - 0.5 = 0.5
-   *   z_eye = z_object / width - z_camera = - z_camera
-   *
-   * From cogl_perspective(), we know that the projection matrix has
-   * the form:
-   *
-   *  (x, 0,  0, 0)
-   *  (0, y,  0, 0)
-   *  (0, 0,  c, d)
-   *  (0, 0, -1, 0)
-   *
-   * Applied to the above, we get clip coordinates of
-   *
-   *  x_clip = x * (- 0.5)
-   *  y_clip = y * 0.5
-   *  w_clip = - 1 * (- z_camera) = z_camera
-   *
-   * Dividing through by w to get normalized device coordinates, we
-   * have, x_nd = x * 0.5 / z_camera, y_nd = - y * 0.5 / z_camera.
-   * The upper left corner of the screen has normalized device coordinates,
-   * (-1, 1), so to have the correct 1:1 mapping, we have to have:
-   *
-   *   z_camera = 0.5 * x = 0.5 * y
-   *
-   * If x != y, then we have a non-uniform aspect ration, and a 1:1 mapping
-   * doesn't make sense.
-   */
-
-  cogl_get_projection_matrix (&projection_matrix);
-  z_camera = 0.5 * projection_matrix.xx;
-
-  modelview_stack =
-    _cogl_framebuffer_get_modelview_stack (_cogl_get_framebuffer ());
-  _cogl_matrix_stack_load_identity (modelview_stack);
-  _cogl_matrix_stack_translate (modelview_stack, -0.5f, -0.5f, -z_camera);
-  _cogl_matrix_stack_scale (modelview_stack,
-                            1.0f / width, -1.0f / height, 1.0f / width);
-  _cogl_matrix_stack_translate (modelview_stack,
-                                0.0f, -1.0 * height, 0.0f);
-}
-
 CoglFeatureFlags
 cogl_get_features (void)
 {
   _COGL_GET_CONTEXT (ctx, 0);
 
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS))
-    ctx->feature_flags &= ~COGL_FEATURE_VBOS;
-
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_PBOS))
-    ctx->feature_flags &= ~COGL_FEATURE_PBOS;
-
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_ARBFP))
-    ctx->feature_flags &= ~COGL_FEATURE_SHADERS_ARBFP;
-
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_GLSL))
-    ctx->feature_flags &= ~COGL_FEATURE_SHADERS_GLSL;
-
   return ctx->feature_flags;
 }
 
@@ -507,9 +440,6 @@ cogl_features_available (CoglFeatureFlags features)
 {
   _COGL_GET_CONTEXT (ctx, 0);
 
-  if (!ctx->features_cached)
-    _cogl_features_init ();
-
   return (ctx->feature_flags & features) == features;
 }
 
@@ -518,9 +448,6 @@ _cogl_features_available_private (CoglFeatureFlagsPrivate features)
 {
   _COGL_GET_CONTEXT (ctx, 0);
 
-  if (!ctx->features_cached)
-    _cogl_features_init ();
-
   return (ctx->feature_flags_private & features) == features;
 }
 
@@ -605,13 +532,14 @@ cogl_flush (void)
 }
 
 void
-cogl_read_pixels (int x,
-                  int y,
-                  int width,
-                  int height,
-                  CoglReadPixelsFlags source,
-                  CoglPixelFormat format,
-                  guint8 *pixels)
+_cogl_read_pixels_with_rowstride (int x,
+                                  int y,
+                                  int width,
+                                  int height,
+                                  CoglReadPixelsFlags source,
+                                  CoglPixelFormat format,
+                                  guint8 *pixels,
+                                  int rowstride)
 {
   CoglFramebuffer *framebuffer;
   int              framebuffer_height;
@@ -621,7 +549,6 @@ cogl_read_pixels (int x,
   GLenum           gl_format;
   GLenum           gl_type;
   CoglPixelFormat  bmp_format;
-  int              rowstride;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -648,22 +575,18 @@ cogl_read_pixels (int x,
 
   /* Initialise the CoglBitmap */
   bpp = _cogl_get_format_bpp (format);
-  rowstride = bpp * width;
   bmp_format = format;
 
   if ((format & COGL_A_BIT))
     {
-      /* FIXME: We are assuming glReadPixels will always give us
-         premultiplied data so we'll set the premult flag on the
-         bitmap format. This will usually be correct because the
-         result of the default blending operations for Cogl ends up
-         with premultiplied data in the framebuffer. However it is
-         possible for the framebuffer to be in whatever format
-         depending on what CoglMaterial is used to render to
-         it. Eventually we may want to add a way for an application to
-         inform Cogl that the framebuffer is not premultiplied in case
-         it is being used for some special purpose. */
-      bmp_format |= COGL_PREMULT_BIT;
+      /* We match the premultiplied state of the target buffer to the
+       * premultiplied state of the framebuffer so that it will get
+       * converted to the right format below */
+
+      if ((framebuffer->format & COGL_PREMULT_BIT))
+        bmp_format |= COGL_PREMULT_BIT;
+      else
+        bmp_format &= ~COGL_PREMULT_BIT;
     }
 
   bmp = _cogl_bitmap_new_from_data (pixels,
@@ -679,15 +602,19 @@ cogl_read_pixels (int x,
      to be more clever and check if the requested type matches that
      but we would need some reliable functions to convert from GL
      types to Cogl types. For now, lets just always read in
-     GL_RGBA/GL_UNSIGNED_BYTE and convert if necessary */
+     GL_RGBA/GL_UNSIGNED_BYTE and convert if necessary. We also need
+     to use this intermediate buffer if the rowstride has padding
+     because GLES does not support setting GL_ROW_LENGTH */
 #ifndef COGL_HAS_GL
-  if (gl_format != GL_RGBA || gl_type != GL_UNSIGNED_BYTE)
+  if (gl_format != GL_RGBA || gl_type != GL_UNSIGNED_BYTE ||
+      rowstride != 4 * width)
     {
       CoglBitmap *tmp_bmp, *dst_bmp;
       guint8 *tmp_data = g_malloc (width * height * 4);
 
       tmp_bmp = _cogl_bitmap_new_from_data (tmp_data,
-                                            COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                            COGL_PIXEL_FORMAT_RGBA_8888 |
+                                            (bmp_format & COGL_PREMULT_BIT),
                                             width, height, 4 * width,
                                             (CoglBitmapDestroyNotify) g_free,
                                             NULL);
@@ -702,7 +629,7 @@ cogl_read_pixels (int x,
          allocating its own buffer so we have to copy the data
          again */
       if ((dst_bmp = _cogl_bitmap_convert_format_and_premult (tmp_bmp,
-                                                              bmp_format)))
+                                                              format)))
         {
           _cogl_bitmap_copy_subregion (dst_bmp,
                                        bmp,
@@ -760,6 +687,21 @@ cogl_read_pixels (int x,
   cogl_object_unref (bmp);
 }
 
+void
+cogl_read_pixels (int x,
+                  int y,
+                  int width,
+                  int height,
+                  CoglReadPixelsFlags source,
+                  CoglPixelFormat format,
+                  guint8 *pixels)
+{
+  _cogl_read_pixels_with_rowstride (x, y, width, height,
+                                    source, format, pixels,
+                                    /* rowstride */
+                                    _cogl_get_format_bpp (format) * width);
+}
+
 static void
 _cogl_disable_other_texcoord_arrays_cb (int texcoord_array_num, gpointer data)
 {
@@ -794,6 +736,7 @@ void
 cogl_begin_gl (void)
 {
   unsigned long enable_flags = 0;
+  CoglPipeline *pipeline;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -814,20 +757,33 @@ cogl_begin_gl (void)
    * projection matrix state
    *
    * NB: _cogl_framebuffer_flush_state may disrupt various state (such
-   * as the material state) when flushing the clip stack, so should
+   * as the pipeline state) when flushing the clip stack, so should
    * always be done first when preparing to draw. */
   _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
 
-  /* Setup the state for the current material */
+  /* Setup the state for the current pipeline */
 
-  /* We considered flushing a specific, minimal material here to try and
+  /* We considered flushing a specific, minimal pipeline here to try and
    * simplify the GL state, but decided to avoid special cases and second
    * guessing what would be actually helpful.
    *
    * A user should instead call cogl_set_source_color4ub() before
    * cogl_begin_gl() to simplify the state flushed.
+   *
+   * XXX: note defining n_tex_coord_attribs using
+   * cogl_pipeline_get_n_layers is a hack, but the problem is that
+   * n_tex_coord_attribs is usually defined when drawing a primitive
+   * which isn't happening here.
+   *
+   * Maybe it would be more useful if this code did flush the
+   * opaque_color_pipeline and then call into cogl-pipeline-opengl.c to then
+   * restore all state for the material's backend back to default OpenGL
+   * values.
    */
-  _cogl_material_flush_gl_state (ctx->source_material, FALSE);
+  pipeline = cogl_get_source ();
+  _cogl_pipeline_flush_gl_state (pipeline,
+                                 FALSE,
+                                 cogl_pipeline_get_n_layers (pipeline));
 
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
@@ -1017,22 +973,115 @@ _cogl_driver_error_quark (void)
   return g_quark_from_static_string ("cogl-driver-error-quark");
 }
 
-void
-cogl_set_source (CoglHandle material_handle)
+typedef struct _CoglSourceState
 {
+  CoglPipeline *pipeline;
+  int push_count;
+} CoglSourceState;
+
+static void
+_push_source_real (CoglPipeline *pipeline)
+{
+  CoglSourceState *top = g_slice_new (CoglSourceState);
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  g_return_if_fail (cogl_is_material (material_handle));
+  top->pipeline = cogl_object_ref (pipeline);
+  top->push_count = 1;
 
-  if (ctx->source_material == material_handle)
+  ctx->source_stack = g_list_prepend (ctx->source_stack, top);
+}
+
+/* FIXME: This should take a context pointer for Cogl 2.0 Technically
+ * we could make it so we can retrieve a context reference from the
+ * pipeline, but this would not by symmetric with cogl_pop_source. */
+void
+cogl_push_source (void *material_or_pipeline)
+{
+  CoglSourceState *top;
+  CoglPipeline *pipeline = COGL_PIPELINE (material_or_pipeline);
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  g_return_if_fail (cogl_is_pipeline (pipeline));
+
+  if (ctx->source_stack)
+    {
+      top = ctx->source_stack->data;
+      if (top->pipeline == pipeline)
+        {
+          top->push_count++;
+          return;
+        }
+      else
+        _push_source_real (pipeline);
+    }
+  else
+    _push_source_real (pipeline);
+}
+
+/* FIXME: This needs to take a context pointer for Cogl 2.0 */
+void
+cogl_pop_source (void)
+{
+  CoglSourceState *top;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  g_return_if_fail (ctx->source_stack);
+
+  top = ctx->source_stack->data;
+  top->push_count--;
+  if (top->push_count == 0)
+    {
+      cogl_object_unref (top->pipeline);
+      g_slice_free (CoglSourceState, top);
+      ctx->source_stack = g_list_delete_link (ctx->source_stack,
+                                              ctx->source_stack);
+    }
+}
+
+/* FIXME: This needs to take a context pointer for Cogl 2.0 */
+void *
+cogl_get_source (void)
+{
+  CoglSourceState *top;
+
+  _COGL_GET_CONTEXT (ctx, NULL);
+
+  g_return_val_if_fail (ctx->source_stack, NULL);
+
+  top = ctx->source_stack->data;
+  return top->pipeline;
+}
+
+void
+cogl_set_source (void *material_or_pipeline)
+{
+  CoglSourceState *top;
+  CoglPipeline *pipeline = COGL_PIPELINE (material_or_pipeline);
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  g_return_if_fail (cogl_is_pipeline (pipeline));
+  g_return_if_fail (ctx->source_stack);
+
+  top = ctx->source_stack->data;
+  if (top->pipeline == pipeline)
     return;
 
-  cogl_handle_ref (material_handle);
-
-  if (ctx->source_material)
-    cogl_handle_unref (ctx->source_material);
-
-  ctx->source_material = material_handle;
+  if (top->push_count == 1)
+    {
+      /* NB: top->pipeline may be only thing keeping pipeline
+       * alive currently so ref pipeline first... */
+      cogl_object_ref (pipeline);
+      cogl_object_unref (top->pipeline);
+      top->pipeline = pipeline;
+    }
+  else
+    {
+      top->push_count--;
+      cogl_push_source (pipeline);
+    }
 }
 
 void
@@ -1042,8 +1091,8 @@ cogl_set_source_texture (CoglHandle texture_handle)
 
   g_return_if_fail (texture_handle != NULL);
 
-  cogl_material_set_layer (ctx->texture_material, 0, texture_handle);
-  cogl_set_source (ctx->texture_material);
+  cogl_pipeline_set_layer_texture (ctx->texture_pipeline, 0, texture_handle);
+  cogl_set_source (ctx->texture_pipeline);
 }
 
 void
