@@ -29,18 +29,27 @@
 
 #define __COGL_H_INSIDE__
 
+#ifdef COGL_ENABLE_EXPERIMENTAL_2_0_API
+#ifndef COGL_ENABLE_EXPERIMENTAL_API
+#define COGL_ENABLE_EXPERIMENTAL_API
+#endif
+#endif
+
 #include <cogl/cogl-defines.h>
 
 #include <cogl/cogl-object.h>
 #include <cogl/cogl-bitmap.h>
 #include <cogl/cogl-color.h>
-#include <cogl/cogl-debug.h>
 #include <cogl/cogl-fixed.h>
-#include <cogl/cogl-material.h>
+#include <cogl/cogl-material-compat.h>
 #include <cogl/cogl-matrix.h>
 #include <cogl/cogl-offscreen.h>
 #include <cogl/cogl-primitives.h>
+#ifdef COGL_ENABLE_EXPERIMENTAL_2_0_API
+#include <cogl/cogl2-path.h>
+#else
 #include <cogl/cogl-path.h>
+#endif
 #include <cogl/cogl-shader.h>
 #include <cogl/cogl-texture.h>
 #include <cogl/cogl-types.h>
@@ -54,6 +63,13 @@
 #include <cogl/cogl-buffer.h>
 #include <cogl/cogl-pixel-array.h>
 #include <cogl/cogl-vector.h>
+#include <cogl/cogl-texture-3d.h>
+#include <cogl/cogl-index-array.h>
+#include <cogl/cogl-vertex-array.h>
+#include <cogl/cogl-indices.h>
+#include <cogl/cogl-vertex-attribute.h>
+#include <cogl/cogl-primitive.h>
+#include <cogl/cogl-pipeline.h>
 #endif
 
 G_BEGIN_DECLS
@@ -290,32 +306,6 @@ cogl_ortho (float left,
             float near,
             float far);
 
-/*
- * _cogl_setup_viewport:
- * @width: Width of the viewport
- * @height: Height of the viewport
- * @fovy: Field of view angle in degrees
- * @aspect: Aspect ratio to determine the field of view along the x-axis
- * @z_near: Nearest visible point along the z-axis
- * @z_far: Furthest visible point along the z-axis
- *
- * Replaces the current viewport and projection matrix with the given
- * values. The viewport is placed at the top left corner of the window
- * with the given width and height. The projection matrix is replaced
- * with one that has a viewing angle of @fovy along the y-axis and a
- * view scaled according to @aspect along the x-axis. The view is
- * clipped according to @z_near and @z_far on the z-axis.
- *
- * This function is used only by Clutter.
- */
-void
-_cogl_setup_viewport (unsigned int width,
-                      unsigned int height,
-                      float fovy,
-                      float aspect,
-                      float z_near,
-                      float z_far);
-
 #ifndef COGL_DISABLE_DEPRECATED
 
 /**
@@ -600,19 +590,61 @@ cogl_clear (const CoglColor *color,
 
 /**
  * cogl_set_source:
- * @material: A #CoglHandle for a material
+ * @material: A #CoglMaterial
  *
- * This function sets the source material that will be used to fill subsequent
- * geometry emitted via the cogl API.
- *
- * <note>In the future we may add the ability to set a front facing material,
- * and a back facing material, in which case this function will set both to the
- * same.</note>
+ * This function changes the material at the top of the source stack.
+ * The material at the top of this stack defines the GPU state used to
+ * process subsequent primitives, such as rectangles drawn with
+ * cogl_rectangle() or vertices drawn using cogl_vertex_buffer_draw().
  *
  * Since: 1.0
  */
 void
-cogl_set_source (CoglHandle material);
+cogl_set_source (void *material);
+
+/**
+ * cogl_get_source:
+ *
+ * Returns the current source material as previously set using
+ * cogl_set_source().
+ *
+ * <note>You should typically consider the returned material immutable
+ * and not try to change any of its properties unless you own a
+ * reference to that material. At times you may be able to get a
+ * reference to an internally managed materials and the result of
+ * modifying such materials is undefined.</note>
+ *
+ * Return value: The current source material.
+ *
+ * Since: 1.6
+ */
+void *
+cogl_get_source (void);
+
+/**
+ * cogl_push_source:
+ * @material: A #CoglMaterial
+ *
+ * Pushes the given @material to the top of the source stack. The
+ * material at the top of this stack defines the GPU state used to
+ * process later primitives as defined by cogl_set_source().
+ *
+ * Since: 1.6
+ */
+void
+cogl_push_source (void *material);
+
+/**
+ * cogl_pop_source:
+ *
+ * Removes the material at the top of the source stack. The material
+ * at the top of this stack defines the GPU state used to process
+ * later primitives as defined by cogl_set_source().
+ *
+ * Since: 1.6
+ */
+void
+cogl_pop_source (void);
 
 /**
  * cogl_set_source_color:
