@@ -32,7 +32,7 @@
 
 #include "clutter-backend.h"
 #include "clutter-debug.h"
-#include "clutter-device-manager.h"
+#include "clutter-device-manager-private.h"
 #include "clutter-private.h"
 
 #ifdef HAVE_XINPUT
