@@ -31,16 +31,14 @@
 #include "clutter-stage-x11.h"
 #include "clutter-x11.h"
 
-#include "../clutter-stage-window.h"
-#include "../clutter-main.h"
-#include "../clutter-feature.h"
-#include "../clutter-color.h"
-#include "../clutter-util.h"
-#include "../clutter-event.h"
-#include "../clutter-enum-types.h"
-#include "../clutter-private.h"
-#include "../clutter-debug.h"
-#include "../clutter-units.h"
+#include "clutter-actor-private.h"
+#include "clutter-debug.h"
+#include "clutter-main.h"
+#include "clutter-feature.h"
+#include "clutter-event.h"
+#include "clutter-enum-types.h"
+#include "clutter-private.h"
+#include "clutter-stage-private.h"
 
 #include "cogl/cogl.h"
 
@@ -557,10 +555,8 @@ update_wm_hints (ClutterStageX11 *stage_x11)
   g_return_if_fail (CLUTTER_IS_BACKEND_X11 (backend));
   backend_x11 = CLUTTER_BACKEND_X11 (backend);
 
-  //wm_hints.flags = StateHint; //MAEMO5
-  wm_hints.flags = StateHint | InputHint; //MAEMO5
+  wm_hints.flags = StateHint;
   wm_hints.initial_state = NormalState;
-  wm_hints.input = True; //MAEMO5
 
   XSetWMHints (backend_x11->xdpy, stage_x11->xwin, &wm_hints);
 }
